@@ -31,6 +31,7 @@ import {
   HttpStatusCode,
   TranslatedString,
   assertUnreachable,
+  createRFC8959AccessTokenEncoded
 } from "@gnu-taler/taler-util";
 import { useEffect } from "preact/hooks";
 import { useSessionState } from "./hooks/session.js";
@@ -121,7 +122,7 @@ function PublicRounting({
           refreshable: true,
         });
       if (resp.type === "ok") {
-        onLoggedUser(username, resp.body.access_token);
+        onLoggedUser(username, createRFC8959AccessTokenEncoded(resp.body.access_token));
       } else {
         switch (resp.case) {
           case HttpStatusCode.Unauthorized:
@@ -394,6 +395,9 @@ function PrivateRouting({
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
+          onCashout={() =>
+            navigateTo(privatePages.home.url({}))
+          }
           onAuthorizationRequired={() =>
             navigateTo(privatePages.solveSecondFactor.url({}))
           }
@@ -461,6 +465,9 @@ function PrivateRouting({
           routeMyAccountDetails={privatePages.myAccountDetails}
           routeMyAccountPassword={privatePages.myAccountPassword}
           routeConversionConfig={privatePages.conversionConfig}
+          onCashout={() =>
+            navigateTo(privatePages.home.url({}))
+          }
           onAuthorizationRequired={() =>
             navigateTo(privatePages.solveSecondFactor.url({}))
           }
@@ -515,6 +522,7 @@ function PrivateRouting({
           onAuthorizationRequired={() =>
             navigateTo(privatePages.solveSecondFactor.url({}))
           }
+          onCashout={() => navigateTo(privatePages.home.url({}))}
           routeClose={privatePages.home}
         />
       );
