@@ -30,7 +30,7 @@ import {
   assertUnreachable,
   buildPayto,
   parsePaytoUri,
-  stringifyPaytoUri
+  stringifyPaytoUri,
 } from "@gnu-taler/taler-util";
 import {
   InternationalizationAPI,
@@ -190,11 +190,7 @@ export function PaytoWireTransferForm({
         amount: sAmount,
       };
       const check = IdempotencyRetry.tryFiveTimes();
-      const resp = await api.createTransaction(
-        credentials,
-        request,
-        check,
-      );
+      const resp = await api.createTransaction(credentials, request, check);
       mutate(() => true);
       if (resp.type === "fail") {
         switch (resp.case) {
@@ -294,78 +290,6 @@ export function PaytoWireTransferForm({
 
   return (
     <div class="grid grid-cols-1 gap-x-8 gap-y-8 pt-6 my-4 md:grid-cols-3 bg-gray-100 px-4 pb-4 rounded-lg">
-      {/* <div class="">
-        <div class="px-2 grid grid-cols-1 gap-y-4 sm:gap-x-4">
-          <label
-            class={
-              "relative flex cursor-pointer rounded-lg border bg-white p-4 shadow-sm focus:outline-none" +
-              (!isRawPayto
-                ? "border-indigo-600 ring-2 ring-indigo-600"
-                : "border-gray-300")
-            }
-          >
-            <input
-              type="radio"
-              name="project-type"
-              value="Newsletter"
-              class="sr-only"
-              aria-labelledby="project-type-0-label"
-              aria-describedby="project-type-0-description-0 project-type-0-description-1"
-              onChange={() => {
-                setIsRawPayto(false);
-              }}
-            />
-            <span class="flex flex-1">
-              <span class="flex flex-col">
-                <span class="block text-sm  font-medium text-gray-900">
-                  <i18n.Translate>Using a form</i18n.Translate>
-                </span>
-              </span>
-            </span>
-          </label>
-
-          {sendingToFixedAccount ? undefined : (
-            <label
-              class={
-                "relative flex cursor-pointer rounded-lg border bg-white p-4 shadow-sm focus:outline-none" +
-                (isRawPayto
-                  ? "border-indigo-600 ring-2 ring-indigo-600"
-                  : "border-gray-300")
-              }
-            >
-              <input
-                type="radio"
-                name="project-type"
-                value="Existing Customers"
-                class="sr-only"
-                aria-labelledby="project-type-1-label"
-                aria-describedby="project-type-1-description-0 project-type-1-description-1"
-                onChange={() => {
-
-                  setIsRawPayto(true);
-                }}
-              />
-              <span class="flex flex-1">
-                <span class="flex flex-col">
-                  <span class="block text-sm font-medium text-gray-900">
-                    <i18n.Translate>Import payto:// URI</i18n.Translate>
-                  </span>
-                </span>
-              </span>
-            </label>
-          )}
-          {routeCashout ? (
-            <a
-              name="do cashout"
-              href={routeCashout.url({})}
-              class="bg-white p-4 rounded-lg text-sm font-semibold leading-6 text-gray-900"
-            >
-              <i18n.Translate>Cashout</i18n.Translate>
-            </a>
-          ) : undefined}
-        </div>
-      </div> */}
-
       <div>
         <fieldset class="px-2 grid grid-cols-1 gap-y-4 sm:gap-x-4">
           <legend class="sr-only">
