@@ -22,12 +22,9 @@
 import {
   AbsoluteTime,
   TalerError,
-  TranslatedString
+  TranslatedString,
 } from "@gnu-taler/taler-util";
-import {
-  urlPattern,
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+import { urlPattern, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { createHashHistory } from "history";
 import { Fragment, VNode, h } from "preact";
 import { Route, Router, route } from "preact-router";
@@ -168,8 +165,7 @@ export function Routing(_p: Props): VNode {
     (AbsoluteTime.isNever(preference.hideMissingAccountUntil) ||
       AbsoluteTime.cmp(now, preference.hideMissingAccountUntil) > 1);
 
-  const shouldLogin =
-    state.status === "loggedOut" || state.status === "expired";
+  const shouldLogin = state.status === "loggedOut";
 
   // function ServerErrorRedirectTo(to: InstancePaths | AdminPaths) {
   //   return function ServerErrorRedirectToImpl(
