@@ -19,11 +19,9 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import {
-  useMerchantApiContext,
-  useTranslationContext,
-} from "@gnu-taler/web-util/browser";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
+import { useSessionContext } from "../../context/session.js";
 import { Entity } from "../../paths/admin/create/CreatePage.js";
 import { Input } from "../form/Input.js";
 import { InputDuration } from "../form/InputDuration.js";
@@ -33,6 +31,7 @@ import { InputLocation } from "../form/InputLocation.js";
 import { InputSelector } from "../form/InputSelector.js";
 import { InputToggle } from "../form/InputToggle.js";
 import { InputWithAddon } from "../form/InputWithAddon.js";
+import { TextField } from "../form/TextField.js";
 
 export function DefaultInstanceFormFields({
   readonlyId,
@@ -42,13 +41,13 @@ export function DefaultInstanceFormFields({
   showId: boolean;
 }): VNode {
   const { i18n } = useTranslationContext();
-  const { url: backendUrl } = useMerchantApiContext();
+  const { state } = useSessionContext();
   return (
     <Fragment>
       {showId && (
         <InputWithAddon<Entity>
           name="id"
-          addonBefore={new URL("instances/", backendUrl.href).href}
+          addonBefore={new URL("instances/", state.backendUrl.href).href}
           readonly={readonlyId}
           label={i18n.str`Identifier`}
           tooltip={i18n.str`Name of the instance in URLs. The 'default' instance is special in that it is used to administer other instances.`}
@@ -61,13 +60,6 @@ export function DefaultInstanceFormFields({
         tooltip={i18n.str`Legal name of the business represented by this instance.`}
       />
 
-      <InputSelector<Entity>
-        name="user_type"
-        label={i18n.str`Type`}
-        tooltip={i18n.str`Different type of account can have different rules and requirements.`}
-        values={["business", "individual"]}
-      />
-
       <Input<Entity>
         name="email"
         label={i18n.str`Email`}
@@ -86,12 +78,6 @@ export function DefaultInstanceFormFields({
         tooltip={i18n.str`Logo image.`}
       />
 
-      <InputToggle<Entity>
-        name="use_stefan"
-        label={i18n.str`Pay transaction fee`}
-        tooltip={i18n.str`Assume the cost of the transaction of let the user pay for it.`}
-      />
-
       <InputGroup
         name="address"
         label={i18n.str`Address`}
@@ -108,6 +94,12 @@ export function DefaultInstanceFormFields({
         <InputLocation name="jurisdiction" />
       </InputGroup>
 
+      <InputToggle<Entity>
+        name="use_stefan"
+        label={i18n.str`Pay transaction fee`}
+        tooltip={i18n.str`Assume the cost of the transaction of let the user pay for it.`}
+      />
+
       <InputDuration<Entity>
         name="default_pay_delay"
         label={i18n.str`Default payment delay`}
