@@ -13,9 +13,6 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
-import {
-  useMerchantApiContext
-} from "@gnu-taler/web-util/browser";
 
 // FIX default import https://github.com/microsoft/TypeScript/issues/49189
 import { AccessToken, OperationOk, TalerHttpError, TalerMerchantApi, TalerMerchantManagementErrorsByMethod, TalerMerchantManagementResultByMethod, opFixedSuccess } from "@gnu-taler/taler-util";
@@ -40,7 +37,7 @@ export function revalidateInstanceProducts() {
 }
 export function useInstanceProducts() {
   const { state: session } = useSessionContext();
-  const { lib: { instance } } = useMerchantApiContext();
+  const { lib: { instance } } = useSessionContext();
 
   const [offset, setOffset] = useState<number | undefined>();
 
@@ -89,7 +86,7 @@ export function revalidateProductDetails() {
 }
 export function useProductDetails(productId: string) {
   const { state: session } = useSessionContext();
-  const { lib: { instance } } = useMerchantApiContext();
+  const { lib: { instance } } = useSessionContext();
 
   async function fetcher([pid, token]: [string, AccessToken]) {
     return await instance.getProductDetails(token, pid);
