@@ -19,7 +19,11 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { Duration, TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  Duration,
+  TalerMerchantApi,
+  createRFC8959AccessTokenPlain,
+} from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { h, VNode } from "preact";
 import { useState } from "preact/hooks";
@@ -33,10 +37,13 @@ import { SetTokenNewInstanceModal } from "../../../components/modal/index.js";
 import { INSTANCE_ID_REGEX } from "../../../utils/constants.js";
 import { undefinedIfEmpty } from "../../../utils/table.js";
 
-export type Entity = Omit<Omit<TalerMerchantApi.InstanceConfigurationMessage, "default_pay_delay">, "default_wire_transfer_delay"> & {
+export type Entity = Omit<
+  Omit<TalerMerchantApi.InstanceConfigurationMessage, "default_pay_delay">,
+  "default_wire_transfer_delay"
+> & {
   auth_token?: string;
-  default_pay_delay: Duration,
-  default_wire_transfer_delay: Duration,
+  default_pay_delay: Duration;
+  default_wire_transfer_delay: Duration;
 };
 
 interface Props {
@@ -90,10 +97,11 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
     default_pay_delay: !value.default_pay_delay
       ? i18n.str`required`
       : !!value.default_wire_transfer_delay &&
-        value.default_wire_transfer_delay.d_ms !== "forever" &&
-        value.default_pay_delay.d_ms !== "forever" &&
-        value.default_pay_delay.d_ms > value.default_wire_transfer_delay.d_ms ?
-        i18n.str`pay delay can't be greater than wire transfer delay` : undefined,
+          value.default_wire_transfer_delay.d_ms !== "forever" &&
+          value.default_pay_delay.d_ms !== "forever" &&
+          value.default_pay_delay.d_ms > value.default_wire_transfer_delay.d_ms
+        ? i18n.str`pay delay can't be greater than wire transfer delay`
+        : undefined,
     default_wire_transfer_delay: !value.default_wire_transfer_delay
       ? i18n.str`required`
       : undefined,
@@ -112,7 +120,7 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
   };
 
   const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
+    (k) => (errors as Record<string, unknown>)[k] !== undefined,
   );
 
   const submit = (): Promise<void> => {
@@ -121,19 +129,26 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
 
     const newToken = newValue.auth_token;
     newValue.auth_token = undefined;
-    newValue.auth = newToken === null || newToken === undefined
-      ? { method: "external" }
-      : { method: "token", token: `secret-token:${newToken}` };
+    newValue.auth =
+      newToken === null || newToken === undefined
+        ? { method: "external" }
+        : { method: "token", token: createRFC8959AccessTokenPlain(newToken) };
     if (!newValue.address) newValue.address = {};
     if (!newValue.jurisdiction) newValue.jurisdiction = {};
     // remove above use conversion
     // schema.validateSync(value, { abortEarly: false })
-    newValue.default_pay_delay = Duration.toTalerProtocolDuration(newValue.default_pay_delay!) as any
-    newValue.default_wire_transfer_delay = Duration.toTalerProtocolDuration(newValue.default_wire_transfer_delay!) as any
+    newValue.default_pay_delay = Duration.toTalerProtocolDuration(
+      newValue.default_pay_delay!,
+    ) as any;
+    newValue.default_wire_transfer_delay = Duration.toTalerProtocolDuration(
+      newValue.default_wire_transfer_delay!,
+    ) as any;
     // delete value.default_pay_delay;
     // delete value.default_wire_transfer_delay;
 
-    return onCreate(newValue as any as TalerMerchantApi.InstanceConfigurationMessage);
+    return onCreate(
+      newValue as any as TalerMerchantApi.InstanceConfigurationMessage,
+    );
   };
 
   function updateToken(token: string | null) {
