@@ -31,6 +31,7 @@ import {
 import { Input } from "../../../../components/form/Input.js";
 import { InputPaytoForm } from "../../../../components/form/InputPaytoForm.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
+import { ImportingAccountModal } from "../../../../components/modal/index.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
 import { safeConvertURL } from "../update/UpdatePage.js";
 
@@ -46,6 +47,7 @@ const accountAuthType = ["none", "basic"];
 export function CreatePage({ onCreate, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
 
+  const [importing, setImporting] = useState(false);
   const [state, setState] = useState<Partial<Entity>>({});
   const facadeURL = safeConvertURL(state.credit_facade_url);
   const errors: FormErrors<Entity> = {
@@ -115,9 +117,25 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
       credit_facade_url,
     });
   };
-
   return (
     <div>
+      {importing && <ImportingAccountModal onCancel={()=> {setImporting(false)}} onConfirm={(ac) => {
+        state.payto_uri = ac.accountURI
+        const u = new URL(ac.infoURL)
+        u.password = ""
+        if (u.username || u.password) {
+          state.credit_facade_credentials = {
+            type: "basic",
+            password: u.password,
+            username: u.username,
+          }
+          state.repeatPassword = u.password
+        }
+        u.password = ""
+        u.username = ""
+        state.credit_facade_url = u.href;
+        setImporting(false)
+      }}  />}
       <section class="section is-main-section">
         <div class="columns">
           <div class="column" />
@@ -171,6 +189,16 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
             </FormProvider>
 
             <div class="buttons is-right mt-5">
+              <button
+                class="button is-info"
+                data-tooltip={i18n.str`Need to complete marked fields`}
+                onClick={() => {
+                  setImporting(true)
+                }}
+              >
+                <i18n.Translate>Import from bank</i18n.Translate>
+              </button>
+
               {onBack && (
                 <button class="button" onClick={onBack}>
                   <i18n.Translate>Cancel</i18n.Translate>
