@@ -27,8 +27,7 @@ import {
   assertUnreachable,
 } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
-  useTranslationContext,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
@@ -43,10 +42,10 @@ import {
   useOrderDetails,
 } from "../../../../hooks/order.js";
 import { Notification } from "../../../../utils/types.js";
+import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { ListPage } from "./ListPage.js";
 import { RefundModal } from "./Table.js";
-import { LoginPage } from "../../../login/index.js";
 
 interface Props {
   onSelect: (id: string) => void;
@@ -65,7 +64,7 @@ export default function OrderList({ onCreate, onSelect }: Props): VNode {
   const result = useInstanceOrders(filter, (d) =>
     setFilter({ ...filter, position: d }),
   );
-  const { lib } = useMerchantApiContext();
+  const { lib } = useSessionContext();
 
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
 
