@@ -52,7 +52,7 @@ export function CardTable({
       <header class="card-header">
         <p class="card-header-title">
           <span class="icon">
-            <i class="mdi mdi-newspaper" />
+            <i class="mdi mdi-lock" />
           </span>
           <i18n.Translate>OTP Devices</i18n.Translate>
         </p>
