@@ -21,7 +21,6 @@
 
 import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
   useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
@@ -36,9 +35,9 @@ import {
   useInstanceProducts
 } from "../../../../hooks/product.js";
 import { Notification } from "../../../../utils/types.js";
+import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { CardTable } from "./Table.js";
-import { LoginPage } from "../../../login/index.js";
 
 interface Props {
   onCreate: () => void;
@@ -49,7 +48,7 @@ export default function ProductList({
   onSelect,
 }: Props): VNode {
   const result = useInstanceProducts();
-  const { lib } = useMerchantApiContext();
+  const { lib } = useSessionContext();
   const { state } = useSessionContext();
   const [deleting, setDeleting] =
     useState<TalerMerchantApi.ProductDetail & WithId | null>(null);
