@@ -15,7 +15,6 @@
  */
 import { HttpStatusCode, TalerError, TalerMerchantApi, TalerMerchantInstanceHttpClient, TalerMerchantManagementResultByMethod, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
   useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
@@ -30,8 +29,8 @@ import {
 } from "../../../hooks/instance.js";
 import { Notification } from "../../../utils/types.js";
 import { LoginPage } from "../../login/index.js";
-import { UpdatePage } from "./UpdatePage.js";
 import { NotFoundPageOrAdminCreate } from "../../notfound/index.js";
+import { UpdatePage } from "./UpdatePage.js";
 
 export interface Props {
   onBack: () => void;
@@ -44,14 +43,14 @@ export interface Props {
 }
 
 export default function Update(props: Props): VNode {
-  const { lib } = useMerchantApiContext();
+  const { lib } = useSessionContext();
   const updateInstance = lib.instance.updateCurrentInstance.bind(lib.instance)
   const result = useInstanceDetails();
   return CommonUpdate(props, result, updateInstance,);
 }
 
 export function AdminUpdate(props: Props & { instanceId: string }): VNode {
-  const { lib } = useMerchantApiContext();
+  const { lib } = useSessionContext();
   const t = lib.subInstanceApi(props.instanceId).instance;
   const updateInstance = t.updateCurrentInstance.bind(t)
   const result = useManagedInstanceDetails(props.instanceId);
