@@ -30,10 +30,11 @@ import {
   TalerAuthenticationHttpClient,
   TalerBankConversionHttpClient,
   TalerCoreBankHttpClient,
-  TalerErrorCode,
   TalerMerchantInstanceHttpClient,
   TalerMerchantManagementHttpClient,
   TransactionsResponse,
+  createRFC8959AccessTokenEncoded,
+  createRFC8959AccessTokenPlain,
   decodeCrock,
   encodeCrock,
   generateIban,
@@ -41,7 +42,6 @@ import {
   randomBytes,
   rsaBlind,
   setGlobalLogLevelFromString,
-  setPrintHttpRequestAsCurl,
   stringifyPayTemplateUri,
 } from "@gnu-taler/taler-util";
 import { clk } from "@gnu-taler/taler-util/clk";
@@ -55,7 +55,8 @@ import {
   WalletApiOperation,
 } from "@gnu-taler/taler-wallet-core";
 import {
-  downloadExchangeInfo, topupReserveWithBank,
+  downloadExchangeInfo,
+  topupReserveWithBank,
 } from "@gnu-taler/taler-wallet-core/dbless";
 import { deepStrictEqual } from "assert";
 import fs from "fs";
@@ -78,7 +79,6 @@ import {
 } from "./harness/helpers.js";
 import { getTestInfo, runTests } from "./integrationtests/testrunner.js";
 import { lintExchangeDeployment } from "./lint.js";
-import { randomUUID } from "crypto";
 
 const logger = new Logger("taler-harness:index.ts");
 
@@ -354,25 +354,46 @@ advancedCli
     );
   });
 
-const configCli = testingCli.subcommand("configArgs", "config", {
-  help: "Subcommands for handling the Taler configuration.",
-});
-
-configCli.subcommand("show", "show").action(async (args) => {
-  const config = Configuration.load();
-  const cfgStr = config.stringify({
-    diagnostics: true,
+const configCli = testingCli
+  .subcommand("configArgs", "config", {
+    help: "Subcommands for handling the Taler configuration.",
+  })
+  .maybeOption("configEntryFile", ["-c", "--config"], clk.STRING, {
+    help: "Configuration file to use.",
+  })
+  .maybeOption("project", ["--project"], clk.STRING, {
+    help: `Selection of the project to inspect/change the config (default: taler).`,
   });
-  console.log(cfgStr);
-});
 
 configCli
-  .subcommand("get", "get")
+  .subcommand("show", "show", {
+    help: "Show the current configuration.",
+  })
+  .action(async (args) => {
+    const config = Configuration.load(
+      args.configArgs.configEntryFile,
+      args.configArgs.project,
+    );
+    const cfgStr = config.stringify({
+      diagnostics: true,
+    });
+    console.log(cfgStr);
+  });
+
+configCli
+  .subcommand("get", "get", {
+    help: "Get a configuration option.",
+  })
   .requiredArgument("section", clk.STRING)
   .requiredArgument("option", clk.STRING)
-  .flag("file", ["-f"])
+  .flag("file", ["-f"], {
+    help: "Treat the value as a filename, expanding placeholders.",
+  })
   .action(async (args) => {
-    const config = Configuration.load();
+    const config = Configuration.load(
+      args.configArgs.configEntryFile,
+      args.configArgs.project,
+    );
     let res;
     if (args.get.file) {
       res = config.getPath(args.get.section, args.get.option);
@@ -387,6 +408,35 @@ configCli
     }
   });
 
+configCli
+  .subcommand("set", "set", {
+    help: "Set a configuration option.",
+  })
+  .requiredArgument("section", clk.STRING)
+  .requiredArgument("option", clk.STRING)
+  .requiredArgument("value", clk.STRING)
+  .flag("dry", ["--dry"], {
+    help: "Do not write the changed config to disk, only write it to stdout.",
+  })
+  .action(async (args) => {
+    const config = Configuration.load(
+      args.configArgs.configEntryFile,
+      args.configArgs.project,
+    );
+    config.setString(args.set.section, args.set.option, args.set.value);
+    if (args.set.dry) {
+      console.log(
+        config.stringify({
+          excludeDefaults: true,
+        }),
+      );
+    } else {
+      config.write({
+        excludeDefaults: true,
+      });
+    }
+  });
+
 const deploymentCli = testingCli.subcommand("deploymentArgs", "deployment", {
   help: "Subcommands for handling GNU Taler deployments.",
 });
@@ -614,7 +664,10 @@ deploymentCli
     },
   )
   .maybeOption("bankToken", ["--bank-admin-token"], clk.STRING, {
-    help: "libeufin bank admin's password if the account creation is restricted",
+    help: "libeufin bank admin's token if the account creation is restricted",
+  })
+  .maybeOption("bankPassword", ["--bank-admin-password"], clk.STRING, {
+    help: "libeufin bank admin's password if the account creation is restricted, it will override --bank-admin-token",
   })
   .requiredOption("name", ["--legal-name"], clk.STRING, {
     help: "legal name of the merchant",
@@ -638,10 +691,13 @@ deploymentCli
     help: "if everything worked ok, change the password of the accounts at the end",
   })
   .action(async (args) => {
-    const managementToken = args.provisionBankMerchant
-      .merchantToken as AccessToken;
-    const bankAdminPassword = args.provisionBankMerchant
-      .bankToken as AccessToken;
+    const managementToken = createRFC8959AccessTokenPlain(
+      args.provisionBankMerchant.merchantToken,
+    );
+    const bankAdminPassword = args.provisionBankMerchant.bankPassword;
+    const bankAdminTokenArg = args.provisionBankMerchant.bankToken
+      ? createRFC8959AccessTokenPlain(args.provisionBankMerchant.bankToken)
+      : undefined;
     const id = args.provisionBankMerchant.id;
     const name = args.provisionBankMerchant.name;
     const email = args.provisionBankMerchant.email;
@@ -694,21 +750,48 @@ deploymentCli
       return;
     }
 
+    let bankAdminToken: AccessToken | undefined;
+    if (bankAdminPassword) {
+      const adminAuth = new TalerAuthenticationHttpClient(
+        bank.getAuthenticationAPI("admin").href,
+        httpLib,
+      );
+
+      const resp = await adminAuth.createAccessTokenBasic(
+        "admin",
+        bankAdminPassword,
+        {
+          scope: "write",
+          duration: {
+            d_us: 1000 * 1000 * 10, //10 secs
+          },
+          refreshable: false,
+        },
+      );
+      if (resp.type === "fail") {
+        logger.error(`could not get bank admin token from password.`);
+        return;
+      }
+      bankAdminToken = resp.body.access_token;
+    } else {
+      bankAdminToken = bankAdminTokenArg;
+    }
+
     /**
      * create bank account
      */
     let accountPayto: PaytoString;
     {
-      const resp = await bank.createAccount(bankAdminPassword, {
+      const resp = await bank.createAccount(bankAdminToken, {
         name: name,
         password: password,
         username: id,
         contact_data:
           email || phone
             ? {
-              email: email,
-              phone: phone,
-            }
+                email: email,
+                phone: phone,
+              }
             : undefined,
       });
 
@@ -730,7 +813,7 @@ deploymentCli
         address: {},
         auth: {
           method: "token",
-          token: `secret-token:${password}`,
+          token: createRFC8959AccessTokenPlain(password),
         },
         default_pay_delay: Duration.toTalerProtocolDuration(
           Duration.fromSpec({ hours: 1 }),
@@ -762,7 +845,7 @@ deploymentCli
      */
     {
       const resp = await merchantInstance.addBankAccount(
-        password as AccessToken,
+        createRFC8959AccessTokenEncoded(password),
         {
           payto_uri: accountPayto,
           credit_facade_url: bank.getRevenueAPI(id).href,
@@ -805,7 +888,7 @@ deploymentCli
 
       {
         const resp = await merchantInstance.addTemplate(
-          password as AccessToken,
+          createRFC8959AccessTokenEncoded(password),
           {
             template_id: "default",
             template_description: "First template",
@@ -817,6 +900,9 @@ deploymentCli
               currency,
               summary: "Pay me!",
             },
+            editable_defaults: {
+              amount: currency,
+            },
           },
         );
         if (resp.type === "fail") {
@@ -832,15 +918,12 @@ deploymentCli
       templateURI = stringifyPayTemplateUri({
         merchantBaseUrl: instanceURL,
         templateId: "default",
-        templateParams: {
-          amount: currency,
-        },
       });
     }
 
     let finalPassword = password;
     if (args.provisionBankMerchant.randomPassword) {
-      const prevPassword = password as AccessToken;
+      const prevPassword = password;
       const randomPassword = encodeCrock(randomBytes(16));
       logger.info("random password: ", randomPassword);
       let token: AccessToken;
@@ -885,10 +968,10 @@ deploymentCli
 
       {
         const resp = await merchantInstance.updateCurrentInstanceAuthentication(
-          prevPassword,
+          createRFC8959AccessTokenEncoded(prevPassword),
           {
             method: "token",
-            token: `secret-token:${randomPassword}` as AccessToken,
+            token: createRFC8959AccessTokenPlain(randomPassword),
           },
         );
         if (resp.type === "fail") {
@@ -902,7 +985,7 @@ deploymentCli
 
       {
         const resp = await merchantInstance.updateBankAccount(
-          randomPassword as AccessToken,
+          createRFC8959AccessTokenEncoded(randomPassword),
           wireAccount,
           {
             credit_facade_url: bank.getRevenueAPI(id).href,
@@ -960,17 +1043,15 @@ deploymentCli
     const httpLib = createPlatformHttpLib({});
     const baseUrl = args.provisionMerchantInstance.merchantApiBaseUrl;
     const api = new TalerMerchantManagementHttpClient(baseUrl, httpLib);
-    const mt = args.provisionMerchantInstance.managementToken;
-    const mtWithoutPrefix = mt.startsWith("secret-token:")
-      ? mt.substring("secret-token:".length)
-      : mt;
-    const managementToken = mtWithoutPrefix as AccessToken;
-
-    const it = args.provisionMerchantInstance.instanceToken;
-    const itWithoutPrefix = it.startsWith("secret-token:")
-      ? it.substring("secret-token:".length)
-      : it;
-    const instanceToken = itWithoutPrefix as AccessToken;
+    const managementToken = createRFC8959AccessTokenEncoded(
+      args.provisionMerchantInstance.managementToken,
+    );
+    const instanceTokenEnc = createRFC8959AccessTokenPlain(
+      args.provisionMerchantInstance.instanceToken,
+    );
+    const instanceTokenPlain = createRFC8959AccessTokenPlain(
+      args.provisionMerchantInstance.instanceToken,
+    );
     const instanceId = args.provisionMerchantInstance.id;
     const instancceName = args.provisionMerchantInstance.name;
     const bankURL = args.provisionMerchantInstance.bankURL;
@@ -982,7 +1063,7 @@ deploymentCli
       address: {},
       auth: {
         method: "token",
-        token: `secret-token:${instanceToken}`,
+        token: instanceTokenPlain,
       },
       default_pay_delay: Duration.toTalerProtocolDuration(
         Duration.fromSpec({ hours: 1 }),
@@ -1005,16 +1086,16 @@ deploymentCli
       process.exit(2);
     }
 
-    const createAccountResp = await api.addBankAccount(instanceToken, {
+    const createAccountResp = await api.addBankAccount(instanceTokenEnc, {
       payto_uri: accountPayto,
       credit_facade_url: bankURL,
       credit_facade_credentials:
         bankUser && bankPassword
           ? {
-            type: "basic",
-            username: bankUser,
-            password: bankPassword,
-          }
+              type: "basic",
+              username: bankUser,
+              password: bankPassword,
+            }
           : undefined,
     });
     if (createAccountResp.type != "ok") {
@@ -1117,23 +1198,6 @@ deploymentCli
     console.log(out);
   });
 
-const deploymentConfigCli = deploymentCli.subcommand("configArgs", "config", {
-  help: "Subcommands the Taler configuration.",
-});
-
-deploymentConfigCli
-  .subcommand("show", "show")
-  .flag("diagnostics", ["-d", "--diagnostics"])
-  .maybeArgument("cfgfile", clk.STRING, {})
-  .action(async (args) => {
-    const cfg = Configuration.load(args.show.cfgfile);
-    console.log(
-      cfg.stringify({
-        diagnostics: args.show.diagnostics,
-      }),
-    );
-  });
-
 testingCli.subcommand("logtest", "logtest").action(async (args) => {
   logger.trace("This is a trace message.");
   logger.info("This is an info message.");
