@@ -26,12 +26,12 @@ import { AmountString, Amounts, CoinStatus } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig, defaultCoinConfig } from "../harness/denomStructures.js";
 import {
-  BankService,
   ExchangeService,
   GlobalTestState,
   MerchantService,
   setupDb,
   generateRandomPayto,
+  FakebankService,
 } from "../harness/harness.js";
 import {
   SimpleTestEnvironmentNg,
@@ -50,7 +50,7 @@ export async function createMyEnvironment(
 ): Promise<SimpleTestEnvironmentNg> {
   const db = await setupDb(t);
 
-  const bank = await BankService.create(t, {
+  const bank = await FakebankService.create(t, {
     allowRegistrations: true,
     currency: "TESTKUDOS",
     database: db.connStr,
