@@ -72,6 +72,8 @@ import { runPaymentTransientTest } from "./test-payment-transient.js";
 import { runPaymentZeroTest } from "./test-payment-zero.js";
 import { runPaymentTest } from "./test-payment.js";
 import { runPaywallFlowTest } from "./test-paywall-flow.js";
+import { runPeerPullLargeTest } from "./test-peer-pull-large.js";
+import { runPeerPushLargeTest } from "./test-peer-push-large.js";
 import { runPeerRepairTest } from "./test-peer-repair.js";
 import { runPeerToPeerPullTest } from "./test-peer-to-peer-pull.js";
 import { runPeerToPeerPushTest } from "./test-peer-to-peer-push.js";
@@ -111,10 +113,13 @@ import { runWalletRefreshTest } from "./test-wallet-refresh.js";
 import { runWalletWirefeesTest } from "./test-wallet-wirefees.js";
 import { runWallettestingTest } from "./test-wallettesting.js";
 import { runWithdrawalAbortBankTest } from "./test-withdrawal-abort-bank.js";
+import { runWithdrawalAmountTest } from "./test-withdrawal-amount.js";
 import { runWithdrawalBankIntegratedTest } from "./test-withdrawal-bank-integrated.js";
 import { runWithdrawalConversionTest } from "./test-withdrawal-conversion.js";
 import { runWithdrawalFakebankTest } from "./test-withdrawal-fakebank.js";
 import { runWithdrawalFeesTest } from "./test-withdrawal-fees.js";
+import { runWithdrawalFlexTest } from "./test-withdrawal-flex.js";
+import { runWithdrawalHandoverTest } from "./test-withdrawal-handover.js";
 import { runWithdrawalHugeTest } from "./test-withdrawal-huge.js";
 import { runWithdrawalManualTest } from "./test-withdrawal-manual.js";
 
@@ -224,6 +229,11 @@ const allTests: TestMainFunction[] = [
   runWalletBlockedPayPeerPullTest,
   runWalletExchangeUpdateTest,
   runWalletRefreshErrorsTest,
+  runPeerPullLargeTest,
+  runPeerPushLargeTest,
+  runWithdrawalHandoverTest,
+  runWithdrawalAmountTest,
+  runWithdrawalFlexTest,
 ];
 
 export interface TestRunSpec {
