@@ -24,7 +24,6 @@ import {
   OperationOk,
   TalerErrorCode,
   codecForChallenge,
-  codecForTalerErrorDetail,
   codecForTanTransmission,
   opKnownAlternativeFailure,
   opKnownHttpFailure,
@@ -185,6 +184,8 @@ export class TalerCoreBankHttpClient {
             return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT:
             return opKnownTalerFailure(details.code, details);
+          case TalerErrorCode.BANK_NON_ADMIN_SET_MIN_CASHOUT:
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_NON_ADMIN_SET_TAN_CHANNEL:
             return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_TAN_CHANNEL_NOT_SUPPORTED:
@@ -281,6 +282,8 @@ export class TalerCoreBankHttpClient {
             return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT:
             return opKnownTalerFailure(details.code, details);
+          case TalerErrorCode.BANK_NON_ADMIN_SET_MIN_CASHOUT:
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_TAN_CHANNEL_NOT_SUPPORTED:
             return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_MISSING_TAN_INFO:
@@ -514,7 +517,7 @@ export class TalerCoreBankHttpClient {
   > {
     const url = new URL(`accounts/${auth.username}/transactions`, this.baseUrl);
     if (idempotencyCheck) {
-      body.request_uid = idempotencyCheck.uid
+      body.request_uid = idempotencyCheck.uid;
     }
     const resp = await this.httpLib.fetch(url.href, {
       method: "POST",
@@ -753,6 +756,8 @@ export class TalerCoreBankHttpClient {
         switch (details.code) {
           case TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED:
             return opKnownTalerFailure(details.code, details);
+          case TalerErrorCode.BANK_CONVERSION_AMOUNT_TO_SMALL:
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_BAD_CONVERSION:
             return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_UNALLOWED_DEBIT:
