@@ -14,7 +14,7 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { HttpRequestLibrary } from "../http-common.js";
+import { HttpRequestLibrary, readTalerErrorResponse } from "../http-common.js";
 import { HttpStatusCode } from "../http-status-codes.js";
 import { createPlatformHttpLib } from "../http.js";
 import { LibtoolVersion } from "../libtool-version.js";
@@ -24,7 +24,7 @@ import {
   opEmptySuccess,
   opKnownHttpFailure,
   opKnownTalerFailure,
-  opSuccess,
+  opSuccessFromHttp,
   opUnknownFailure,
 } from "../operation.js";
 import { TalerErrorCode } from "../taler-error-codes.js";
@@ -77,9 +77,9 @@ export class TalerBankIntegrationHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForIntegrationBankConfig());
+        return opSuccessFromHttp(resp, codecForIntegrationBankConfig());
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -106,11 +106,11 @@ export class TalerBankIntegrationHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForBankWithdrawalOperationStatus());
+        return opSuccessFromHttp(resp, codecForBankWithdrawalOperationStatus());
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -129,27 +129,30 @@ export class TalerBankIntegrationHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForBankWithdrawalOperationPostResponse());
+        return opSuccessFromHttp(
+          resp,
+          codecForBankWithdrawalOperationPostResponse(),
+        );
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict: {
-        const body = await resp.json();
+        const body = await readTalerErrorResponse(resp);
         const details = codecForTalerErrorDetail().decode(body);
         switch (details.code) {
           case TalerErrorCode.BANK_WITHDRAWAL_OPERATION_RESERVE_SELECTION_CONFLICT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_DUPLICATE_RESERVE_PUB_SUBJECT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_UNKNOWN_ACCOUNT:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_ACCOUNT_IS_NOT_EXCHANGE:
-            return opKnownTalerFailure(details.code, resp);
+            return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, body);
+            return opUnknownFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -170,7 +173,7 @@ export class TalerBankIntegrationHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 }
