@@ -14,7 +14,7 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { HttpRequestLibrary, makeBasicAuthHeader } from "../http-common.js";
+import { HttpRequestLibrary, makeBasicAuthHeader, readTalerErrorResponse } from "../http-common.js";
 import { HttpStatusCode } from "../http-status-codes.js";
 import { createPlatformHttpLib } from "../http.js";
 import {
@@ -22,7 +22,7 @@ import {
   ResultByMethod,
   opFixedSuccess,
   opKnownHttpFailure,
-  opSuccess,
+  opSuccessFromHttp,
   opUnknownFailure,
 } from "../operation.js";
 import {
@@ -77,7 +77,7 @@ export class TalerWireGatewayHttpClient {
   //   });
   //   switch (resp.status) {
   //     case HttpStatusCode.Ok: return opSuccess(resp, codecForCoreBankConfig())
-  //     default: return opUnknownFailure(resp, await resp.text())
+  //     default: return opUnknownFailure(resp, await readTalerErrorResponse(resp))
   //   }
   // }
 
@@ -96,7 +96,7 @@ export class TalerWireGatewayHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForTransferResponse());
+        return opSuccessFromHttp(resp, codecForTransferResponse());
       //FIXME: show more details in docs
       case HttpStatusCode.BadRequest:
         return opKnownHttpFailure(resp.status, resp);
@@ -108,7 +108,7 @@ export class TalerWireGatewayHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -131,10 +131,10 @@ export class TalerWireGatewayHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForIncomingHistory());
+        return opSuccessFromHttp(resp, codecForIncomingHistory());
       //FIXME: account should not be returned or make it optional
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, {
+        return opFixedSuccess({
           incoming_transactions: [],
           credit_account: undefined,
         });
@@ -147,7 +147,7 @@ export class TalerWireGatewayHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -170,10 +170,10 @@ export class TalerWireGatewayHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForOutgoingHistory());
+        return opSuccessFromHttp(resp, codecForOutgoingHistory());
       //FIXME: account should not be returned or make it optional
       case HttpStatusCode.NoContent:
-        return opFixedSuccess(resp, {
+        return opFixedSuccess({
           outgoing_transactions: [],
           debit_account: undefined,
         });
@@ -186,7 +186,7 @@ export class TalerWireGatewayHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 
@@ -208,7 +208,7 @@ export class TalerWireGatewayHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccess(resp, codecForAddIncomingResponse());
+        return opSuccessFromHttp(resp, codecForAddIncomingResponse());
       //FIXME: show more details in docs
       case HttpStatusCode.BadRequest:
         return opKnownHttpFailure(resp.status, resp);
@@ -220,7 +220,7 @@ export class TalerWireGatewayHttpClient {
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await resp.text());
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
     }
   }
 }
