@@ -18,17 +18,18 @@ export * from "./contract-terms.js";
 export * from "./errors.js";
 export { fnutil } from "./fnutils.js";
 export * from "./helpers.js";
-export * from "./http-client/bank-conversion.js";
 export * from "./http-client/authentication.js";
+export * from "./http-client/bank-conversion.js";
 export * from "./http-client/bank-core.js";
-export * from "./http-client/merchant.js";
 export * from "./http-client/bank-integration.js";
 export * from "./http-client/bank-revenue.js";
 export * from "./http-client/bank-wire.js";
+export * from "./http-client/challenger.js";
 export * from "./http-client/exchange.js";
-export { CacheEvictor } from "./http-client/utils.js";
+export * from "./http-client/merchant.js";
 export * from "./http-client/officer-account.js";
 export * from "./http-client/types.js";
+export { CacheEvictor } from "./http-client/utils.js";
 export * from "./http-status-codes.js";
 export * from "./i18n.js";
 export * from "./iban.js";
@@ -37,7 +38,6 @@ export * from "./kdf.js";
 export * from "./libeufin-api-types.js";
 export * from "./libtool-version.js";
 export * from "./logging.js";
-export * from "./merchant-api-types.js";
 export {
   crypto_sign_keyPair_fromSeed,
   randomBytes,
