@@ -30,8 +30,12 @@ export enum NotificationType {
   BalanceChange = "balance-change",
   BackupOperationError = "backup-error",
   TransactionStateTransition = "transaction-state-transition",
+  /**
+   * @deprecated
+   */
   WithdrawalOperationTransition = "withdrawal-operation-transition",
   ExchangeStateTransition = "exchange-state-transition",
+  Idle = "idle",
   TaskObservabilityEvent = "task-observability-event",
   RequestObservabilityEvent = "request-observability-event",
 }
@@ -124,7 +128,7 @@ export enum ObservabilityEventType {
   TaskStart = "task-start",
   TaskStop = "task-stop",
   TaskReset = "task-reset",
-  ShepherdTaskResult = "sheperd-task-result",
+  ShepherdTaskResult = "shepherd-task-result",
   DeclareTaskDependency = "declare-task-dependency",
   CryptoStart = "crypto-start",
   CryptoFinishSuccess = "crypto-finish-success",
@@ -230,6 +234,10 @@ export interface WithdrawalOperationTransitionNotification {
   uri: string;
 }
 
+export interface IdleNotification {
+  type: NotificationType.Idle;
+}
+
 export type WalletNotification =
   | BalanceChangeNotification
   | WithdrawalOperationTransitionNotification
@@ -237,4 +245,5 @@ export type WalletNotification =
   | ExchangeStateTransitionNotification
   | TransactionStateTransitionNotification
   | TaskProgressNotification
-  | RequestProgressNotification;
+  | RequestProgressNotification
+  | IdleNotification;
