@@ -151,6 +151,7 @@ export enum TransactionMinorState {
   RefundAvailable = "refund-available",
   AcceptRefund = "accept-refund",
   PaidByOther = "paid-by-other",
+  CompletedByOtherWallet = "completed-by-other-wallet",
 }
 
 export enum TransactionAction {
@@ -214,7 +215,6 @@ export type Transaction =
   | TransactionWithdrawal
   | TransactionPayment
   | TransactionRefund
-  | TransactionReward
   | TransactionRefresh
   | TransactionDeposit
   | TransactionPeerPullCredit
@@ -231,7 +231,6 @@ export enum TransactionType {
   Payment = "payment",
   Refund = "refund",
   Refresh = "refresh",
-  Reward = "reward",
   Deposit = "deposit",
   PeerPushDebit = "peer-push-debit",
   PeerPushCredit = "peer-push-credit",
@@ -641,23 +640,6 @@ export interface TransactionRefund extends TransactionCommon {
   paymentInfo: RefundPaymentInfo | undefined;
 }
 
-export interface TransactionReward extends TransactionCommon {
-  type: TransactionType.Reward;
-
-  // Raw amount of the tip, without extra fees that apply
-  amountRaw: AmountString;
-
-  /**
-   * More information about the merchant
-   */
-  // merchant: MerchantInfo;
-
-  // Amount will be (or was) added to the wallet's balance after fees and refreshing
-  amountEffective: AmountString;
-
-  merchantBaseUrl: string;
-}
-
 /**
  * A transaction shown for refreshes.
  * Only shown for (1) refreshes not associated with other transactions
