@@ -268,14 +268,16 @@ export async function selectPayCoins(
   }
 
   return await wex.db.runReadOnlyTx(
-    [
-      "coinAvailability",
-      "denominations",
-      "refreshGroups",
-      "exchanges",
-      "exchangeDetails",
-      "coins",
-    ],
+    {
+      storeNames: [
+        "coinAvailability",
+        "denominations",
+        "refreshGroups",
+        "exchanges",
+        "exchangeDetails",
+        "coins",
+      ],
+    },
     async (tx) => {
       const materialAvSel = await internalSelectPayCoins(wex, tx, req, false);
 
@@ -689,7 +691,7 @@ export function checkAccountRestriction(
     switch (myRestriction.type) {
       case "deny":
         return { ok: false };
-      case "regex":
+      case "regex": {
         const regex = new RegExp(myRestriction.payto_regex);
         if (!regex.test(paytoUri)) {
           return {
@@ -698,6 +700,7 @@ export function checkAccountRestriction(
             hintI18n: myRestriction.human_hint_i18n,
           };
         }
+      }
     }
   }
   return {
@@ -907,7 +910,7 @@ async function selectPayCandidates(
         coinAvail.exchangeBaseUrl,
         coinAvail.denomPubHash,
       ]);
-      checkDbInvariant(!!denom);
+      checkDbInvariant(!!denom, `denomination of a coin is missing hash: ${coinAvail.denomPubHash}`);
       if (denom.isRevoked) {
         logger.trace("denom is revoked");
         continue;
@@ -1140,15 +1143,17 @@ export async function selectPeerCoins(
   }
 
   return await wex.db.runReadWriteTx(
-    [
-      "exchanges",
-      "contractTerms",
-      "coins",
-      "coinAvailability",
-      "denominations",
-      "refreshGroups",
-      "exchangeDetails",
-    ],
+    {
+      storeNames: [
+        "exchanges",
+        "contractTerms",
+        "coins",
+        "coinAvailability",
+        "denominations",
+        "refreshGroups",
+        "exchangeDetails",
+      ],
+    },
     async (tx): Promise<SelectPeerCoinsResult> => {
       const exchanges = await tx.exchanges.iter().toArray();
       const currency = Amounts.currencyOf(instructedAmount);
