@@ -21,6 +21,7 @@ import {
   AbsoluteTime,
   AmountJson,
   Amounts,
+  AsyncFlag,
   CoinRefreshRequest,
   CoinStatus,
   Duration,
@@ -35,6 +36,7 @@ import {
   TalerProtocolTimestamp,
   TombstoneIdStr,
   TransactionIdStr,
+  WalletNotification,
   assertUnreachable,
   checkDbInvariant,
   checkLogicInvariant,
@@ -119,7 +121,7 @@ export async function makeCoinAvailable(
     coinRecord.exchangeBaseUrl,
     coinRecord.denomPubHash,
   ]);
-  checkDbInvariant(!!denom);
+  checkDbInvariant(!!denom, `denomination of a coin is missing hash: ${coinRecord.denomPubHash}`);
   const ageRestriction = coinRecord.maxAge;
   let car = await tx.coinAvailability.get([
     coinRecord.exchangeBaseUrl,
@@ -173,13 +175,13 @@ export async function spendCoins(
       coin.exchangeBaseUrl,
       coin.denomPubHash,
     );
-    checkDbInvariant(!!denom);
+    checkDbInvariant(!!denom, `denomination of a coin is missing hash: ${coin.denomPubHash}`);
     const coinAvailability = await tx.coinAvailability.get([
       coin.exchangeBaseUrl,
       coin.denomPubHash,
       coin.maxAge,
     ]);
-    checkDbInvariant(!!coinAvailability);
+    checkDbInvariant(!!coinAvailability, `age denom info is missing for ${coin.maxAge}`);
     const contrib = csi.contributions[i];
     if (coin.status !== CoinStatus.Fresh) {
       const alloc = coin.spendAllocation;
@@ -211,7 +213,6 @@ export async function spendCoins(
       amount: Amounts.stringify(remaining.amount),
       coinPub: coin.coinPub,
     });
-    checkDbInvariant(!!coinAvailability);
     if (coinAvailability.freshCoinCount === 0) {
       throw Error(
         `invalid coin count ${coinAvailability.freshCoinCount} in DB`,
@@ -555,6 +556,28 @@ export function getAutoRefreshExecuteThreshold(d: {
   return AbsoluteTime.addDuration(expireWithdraw, deltaDiv);
 }
 
+/**
+ * Type and schema definitions for pending tasks in the wallet.
+ *
+ * These are only used internally, and are not part of the stable public
+ * interface to the wallet.
+ */
+
+export enum PendingTaskType {
+  ExchangeUpdate = "exchange-update",
+  Purchase = "purchase",
+  Refresh = "refresh",
+  Recoup = "recoup",
+  RewardPickup = "reward-pickup",
+  Withdraw = "withdraw",
+  Deposit = "deposit",
+  Backup = "backup",
+  PeerPushDebit = "peer-push-debit",
+  PeerPullCredit = "peer-pull-credit",
+  PeerPushCredit = "peer-push-credit",
+  PeerPullDebit = "peer-pull-debit",
+}
+
 /**
  * Parsed representation of task identifiers.
  */
@@ -745,27 +768,56 @@ export interface TransactionContext {
   deleteTransaction(): Promise<void>;
 }
 
-/**
- * Type and schema definitions for pending tasks in the wallet.
- *
- * These are only used internally, and are not part of the stable public
- * interface to the wallet.
- */
-
-export enum PendingTaskType {
-  ExchangeUpdate = "exchange-update",
-  Purchase = "purchase",
-  Refresh = "refresh",
-  Recoup = "recoup",
-  RewardPickup = "reward-pickup",
-  Withdraw = "withdraw",
-  Deposit = "deposit",
-  Backup = "backup",
-  PeerPushDebit = "peer-push-debit",
-  PeerPullCredit = "peer-pull-credit",
-  PeerPushCredit = "peer-push-credit",
-  PeerPullDebit = "peer-pull-debit",
-}
-
 declare const __taskIdStr: unique symbol;
 export type TaskIdStr = string & { [__taskIdStr]: true };
+
+/**
+ * Wait until the wallet is in a particular state.
+ *
+ * Two functions must be provided:
+ * 1. checkState, which checks if the wallet is in the
+ *    desired state.
+ * 2. filterNotification, which checks whether a notification
+ *    might have lead to a state change.
+ */
+export async function genericWaitForState(
+  wex: WalletExecutionContext,
+  args: {
+    checkState: () => Promise<boolean>;
+    filterNotification: (notif: WalletNotification) => boolean;
+  },
+): Promise<void> {
+  await wex.taskScheduler.ensureRunning();
+
+  // FIXME: Clean up using the new JS "using" / Symbol.dispose syntax.
+  const flag = new AsyncFlag();
+  // Raise purchaseNotifFlag whenever we get a notification
+  // about our refresh.
+  const cancelNotif = wex.ws.addNotificationListener((notif) => {
+    if (args.filterNotification(notif)) {
+      flag.raise();
+    }
+  });
+  const unregisterOnCancelled = wex.cancellationToken.onCancelled(() => {
+    cancelNotif();
+    flag.raise();
+  });
+
+  try {
+    while (true) {
+      if (wex.cancellationToken.isCancelled) {
+        throw Error("cancelled");
+      }
+      if (await args.checkState()) {
+        return;
+      }
+      // Wait for the next transition
+      await flag.wait();
+      flag.reset();
+    }
+  } catch (e) {
+    unregisterOnCancelled();
+    cancelNotif();
+    throw e;
+  }
+}
