@@ -20,6 +20,7 @@ import {
   CheckPeerPushDebitResponse,
   CoinRefreshRequest,
   ContractTermsUtil,
+  ExchangePurseDeposits,
   HttpStatusCode,
   InitiatePeerPushDebitRequest,
   InitiatePeerPushDebitResponse,
@@ -103,19 +104,22 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
 
   async deleteTransaction(): Promise<void> {
     const { wex, pursePub, transactionId } = this;
-    await wex.db.runReadWriteTx(["peerPushDebit", "tombstones"], async (tx) => {
-      const debit = await tx.peerPushDebit.get(pursePub);
-      if (debit) {
-        await tx.peerPushDebit.delete(pursePub);
-        await tx.tombstones.put({ id: transactionId });
-      }
-    });
+    await wex.db.runReadWriteTx(
+      { storeNames: ["peerPushDebit", "tombstones"] },
+      async (tx) => {
+        const debit = await tx.peerPushDebit.get(pursePub);
+        if (debit) {
+          await tx.peerPushDebit.delete(pursePub);
+          await tx.tombstones.put({ id: transactionId });
+        }
+      },
+    );
   }
 
   async suspendTransaction(): Promise<void> {
     const { wex, pursePub, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushDebit"],
+      { storeNames: ["peerPushDebit"] },
       async (tx) => {
         const pushDebitRec = await tx.peerPushDebit.get(pursePub);
         if (!pushDebitRec) {
@@ -173,7 +177,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
   async abortTransaction(): Promise<void> {
     const { wex, pursePub, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushDebit"],
+      { storeNames: ["peerPushDebit"] },
       async (tx) => {
         const pushDebitRec = await tx.peerPushDebit.get(pursePub);
         if (!pushDebitRec) {
@@ -227,7 +231,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
   async resumeTransaction(): Promise<void> {
     const { wex, pursePub, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushDebit"],
+      { storeNames: ["peerPushDebit"] },
       async (tx) => {
         const pushDebitRec = await tx.peerPushDebit.get(pursePub);
         if (!pushDebitRec) {
@@ -285,7 +289,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
   async failTransaction(): Promise<void> {
     const { wex, pursePub, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      ["peerPushDebit"],
+      { storeNames: ["peerPushDebit"] },
       async (tx) => {
         const pushDebitRec = await tx.peerPushDebit.get(pursePub);
         if (!pushDebitRec) {
@@ -402,7 +406,7 @@ async function handlePurseCreationConflict(
   const instructedAmount = Amounts.parseOrThrow(peerPushInitiation.amount);
   const sel = peerPushInitiation.coinSel;
 
-  checkDbInvariant(!!sel);
+  checkDbInvariant(!!sel, `no coin selected for peer push initiation ${peerPushInitiation.pursePub}`);
 
   const repair: PreviousPayCoins = [];
 
@@ -433,7 +437,7 @@ async function handlePurseCreationConflict(
       assertUnreachable(coinSelRes);
   }
 
-  await wex.db.runReadWriteTx(["peerPushDebit"], async (tx) => {
+  await wex.db.runReadWriteTx({ storeNames: ["peerPushDebit"] }, async (tx) => {
     const myPpi = await tx.peerPushDebit.get(peerPushInitiation.pursePub);
     if (!myPpi) {
       return;
@@ -469,7 +473,7 @@ async function processPeerPushDebitCreateReserve(
   logger.trace(`processing ${transactionId} pending(create-reserve)`);
 
   const contractTermsRecord = await wex.db.runReadOnlyTx(
-    ["contractTerms"],
+    { storeNames: ["contractTerms"] },
     async (tx) => {
       return tx.contractTerms.get(hContractTerms);
     },
@@ -502,16 +506,18 @@ async function processPeerPushDebitCreateReserve(
         assertUnreachable(coinSelRes);
     }
     const transitionDone = await wex.db.runReadWriteTx(
-      [
-        "exchanges",
-        "contractTerms",
-        "coins",
-        "coinAvailability",
-        "denominations",
-        "refreshGroups",
-        "refreshSessions",
-        "peerPushDebit",
-      ],
+      {
+        storeNames: [
+          "exchanges",
+          "contractTerms",
+          "coins",
+          "coinAvailability",
+          "denominations",
+          "refreshGroups",
+          "refreshSessions",
+          "peerPushDebit",
+        ],
+      },
       async (tx) => {
         const ppi = await tx.peerPushDebit.get(pursePub);
         if (!ppi) {
@@ -564,12 +570,6 @@ async function processPeerPushDebitCreateReserve(
     peerPushInitiation.coinSel,
   );
 
-  const depositSigsResp = await wex.cryptoApi.signPurseDeposits({
-    exchangeBaseUrl: peerPushInitiation.exchangeBaseUrl,
-    pursePub: peerPushInitiation.pursePub,
-    coins,
-  });
-
   const encryptContractRequest: EncryptContractRequest = {
     contractTerms: contractTermsRecord.contractTermsRaw,
     mergePriv: peerPushInitiation.mergePriv,
@@ -580,66 +580,115 @@ async function processPeerPushDebitCreateReserve(
     nonce: peerPushInitiation.contractEncNonce,
   };
 
-  logger.trace(`encrypt contract request: ${j2s(encryptContractRequest)}`);
-
   const econtractResp = await wex.cryptoApi.encryptContractForMerge(
     encryptContractRequest,
   );
 
-  const createPurseUrl = new URL(
-    `purses/${peerPushInitiation.pursePub}/create`,
-    peerPushInitiation.exchangeBaseUrl,
-  );
+  const maxBatchSize = 100;
 
-  const reqBody = {
-    amount: peerPushInitiation.amount,
-    merge_pub: peerPushInitiation.mergePub,
-    purse_sig: purseSigResp.sig,
-    h_contract_terms: hContractTerms,
-    purse_expiration: timestampProtocolFromDb(purseExpiration),
-    deposits: depositSigsResp.deposits,
-    min_age: 0,
-    econtract: econtractResp.econtract,
-  };
+  for (let i = 0; i < coins.length; i += maxBatchSize) {
+    const batchSize = Math.min(maxBatchSize, coins.length - i);
+    const batchCoins = coins.slice(i, i + batchSize);
 
-  logger.trace(`request body: ${j2s(reqBody)}`);
+    const depositSigsResp = await wex.cryptoApi.signPurseDeposits({
+      exchangeBaseUrl: peerPushInitiation.exchangeBaseUrl,
+      pursePub: peerPushInitiation.pursePub,
+      coins: batchCoins,
+    });
 
-  const httpResp = await wex.http.fetch(createPurseUrl.href, {
-    method: "POST",
-    body: reqBody,
-    cancellationToken: wex.cancellationToken,
-  });
+    if (i == 0) {
+      // First batch creates the purse!
 
-  {
-    const resp = await httpResp.json();
-    logger.info(`resp: ${j2s(resp)}`);
-  }
+      logger.trace(`encrypt contract request: ${j2s(encryptContractRequest)}`);
 
-  switch (httpResp.status) {
-    case HttpStatusCode.Ok:
-      break;
-    case HttpStatusCode.Forbidden: {
-      // FIXME: Store this error!
-      await ctx.failTransaction();
-      return TaskRunResult.finished();
-    }
-    case HttpStatusCode.Conflict: {
-      // Handle double-spending
-      return handlePurseCreationConflict(wex, peerPushInitiation, httpResp);
-    }
-    default: {
-      const errResp = await readTalerErrorResponse(httpResp);
-      return {
-        type: TaskRunResultType.Error,
-        errorDetail: errResp,
+      const createPurseUrl = new URL(
+        `purses/${peerPushInitiation.pursePub}/create`,
+        peerPushInitiation.exchangeBaseUrl,
+      );
+
+      const reqBody = {
+        amount: peerPushInitiation.amount,
+        merge_pub: peerPushInitiation.mergePub,
+        purse_sig: purseSigResp.sig,
+        h_contract_terms: hContractTerms,
+        purse_expiration: timestampProtocolFromDb(purseExpiration),
+        deposits: depositSigsResp.deposits,
+        min_age: 0,
+        econtract: econtractResp.econtract,
       };
+
+      if (logger.shouldLogTrace()) {
+        logger.trace(`request body: ${j2s(reqBody)}`);
+      }
+
+      const httpResp = await wex.http.fetch(createPurseUrl.href, {
+        method: "POST",
+        body: reqBody,
+        cancellationToken: wex.cancellationToken,
+      });
+
+      switch (httpResp.status) {
+        case HttpStatusCode.Ok:
+          // Possibly on to the next batch.
+          continue;
+        case HttpStatusCode.Forbidden: {
+          // FIXME: Store this error!
+          await ctx.failTransaction();
+          return TaskRunResult.finished();
+        }
+        case HttpStatusCode.Conflict: {
+          // Handle double-spending
+          return handlePurseCreationConflict(wex, peerPushInitiation, httpResp);
+        }
+        default: {
+          const errResp = await readTalerErrorResponse(httpResp);
+          return {
+            type: TaskRunResultType.Error,
+            errorDetail: errResp,
+          };
+        }
+      }
+    } else {
+      const purseDepositUrl = new URL(
+        `purses/${pursePub}/deposit`,
+        peerPushInitiation.exchangeBaseUrl,
+      );
+
+      const depositPayload: ExchangePurseDeposits = {
+        deposits: depositSigsResp.deposits,
+      };
+
+      const httpResp = await wex.http.fetch(purseDepositUrl.href, {
+        method: "POST",
+        body: depositPayload,
+        cancellationToken: wex.cancellationToken,
+      });
+
+      switch (httpResp.status) {
+        case HttpStatusCode.Ok:
+          // Possibly on to the next batch.
+          continue;
+        case HttpStatusCode.Forbidden: {
+          // FIXME: Store this error!
+          await ctx.failTransaction();
+          return TaskRunResult.finished();
+        }
+        case HttpStatusCode.Conflict: {
+          // Handle double-spending
+          return handlePurseCreationConflict(wex, peerPushInitiation, httpResp);
+        }
+        default: {
+          const errResp = await readTalerErrorResponse(httpResp);
+          return {
+            type: TaskRunResultType.Error,
+            errorDetail: errResp,
+          };
+        }
+      }
     }
   }
 
-  if (httpResp.status !== HttpStatusCode.Ok) {
-    // FIXME: do proper error reporting
-    throw Error("got error response from exchange");
-  }
+  // All batches done!
 
   await transitionPeerPushDebitTransaction(wex, pursePub, {
     stFrom: PeerPushDebitStatus.PendingCreatePurse,
@@ -676,14 +725,16 @@ async function processPeerPushDebitAbortingDeletePurse(
   logger.info(`deleted purse with response status ${resp.status}`);
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    [
-      "peerPushDebit",
-      "refreshGroups",
-      "refreshSessions",
-      "denominations",
-      "coinAvailability",
-      "coins",
-    ],
+    {
+      storeNames: [
+        "peerPushDebit",
+        "refreshGroups",
+        "refreshSessions",
+        "denominations",
+        "coinAvailability",
+        "coins",
+      ],
+    },
     async (tx) => {
       const ppiRec = await tx.peerPushDebit.get(pursePub);
       if (!ppiRec) {
@@ -735,6 +786,7 @@ interface SimpleTransition {
   stTo: PeerPushDebitStatus;
 }
 
+// FIXME: This should be a transition on the peer push debit transaction context!
 async function transitionPeerPushDebitTransaction(
   wex: WalletExecutionContext,
   pursePub: string,
@@ -745,7 +797,7 @@ async function transitionPeerPushDebitTransaction(
     pursePub,
   });
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["peerPushDebit"],
+    { storeNames: ["peerPushDebit"] },
     async (tx) => {
       const ppiRec = await tx.peerPushDebit.get(pursePub);
       if (!ppiRec) {
@@ -782,7 +834,7 @@ async function processPeerPushDebitAbortingRefreshDeleted(
     await waitRefreshFinal(wex, peerPushInitiation.abortRefreshGroupId);
   }
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["refreshGroups", "peerPushDebit"],
+    { storeNames: ["refreshGroups", "peerPushDebit"] },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(abortRefreshGroupId);
       let newOpState: PeerPushDebitStatus | undefined;
@@ -831,7 +883,7 @@ async function processPeerPushDebitAbortingRefreshExpired(
     pursePub: peerPushInitiation.pursePub,
   });
   const transitionInfo = await wex.db.runReadWriteTx(
-    ["peerPushDebit", "refreshGroups"],
+    { storeNames: ["peerPushDebit", "refreshGroups"] },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(abortRefreshGroupId);
       let newOpState: PeerPushDebitStatus | undefined;
@@ -914,14 +966,16 @@ async function processPeerPushDebitReady(
   } else if (resp.status === HttpStatusCode.Gone) {
     logger.info(`purse ${pursePub} is gone, aborting peer-push-debit`);
     const transitionInfo = await wex.db.runReadWriteTx(
-      [
-        "peerPushDebit",
-        "refreshGroups",
-        "refreshSessions",
-        "denominations",
-        "coinAvailability",
-        "coins",
-      ],
+      {
+        storeNames: [
+          "peerPushDebit",
+          "refreshGroups",
+          "refreshSessions",
+          "denominations",
+          "coinAvailability",
+          "coins",
+        ],
+      },
       async (tx) => {
         const ppiRec = await tx.peerPushDebit.get(pursePub);
         if (!ppiRec) {
@@ -975,7 +1029,7 @@ export async function processPeerPushDebit(
   pursePub: string,
 ): Promise<TaskRunResult> {
   const peerPushInitiation = await wex.db.runReadOnlyTx(
-    ["peerPushDebit"],
+    { storeNames: ["peerPushDebit"] },
     async (tx) => {
       return tx.peerPushDebit.get(pursePub);
     },
@@ -1074,16 +1128,18 @@ export async function initiatePeerPushDebit(
   const contractEncNonce = encodeCrock(getRandomBytes(24));
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    [
-      "exchanges",
-      "contractTerms",
-      "coins",
-      "coinAvailability",
-      "denominations",
-      "refreshGroups",
-      "refreshSessions",
-      "peerPushDebit",
-    ],
+    {
+      storeNames: [
+        "exchanges",
+        "contractTerms",
+        "coins",
+        "coinAvailability",
+        "denominations",
+        "refreshGroups",
+        "refreshSessions",
+        "peerPushDebit",
+      ],
+    },
     async (tx) => {
       const ppi: PeerPushDebitRecord = {
         amount: Amounts.stringify(instructedAmount),
