@@ -129,7 +129,6 @@ import {
   computeRefreshTransactionState,
   RefreshTransactionContext,
 } from "./refresh.js";
-import { RewardTransactionContext } from "./reward.js";
 import type { WalletExecutionContext } from "./wallet.js";
 import {
   augmentPaytoUrisForWithdrawal,
@@ -199,7 +198,6 @@ function shouldSkipSearch(
  */
 const txOrder: { [t in TransactionType]: number } = {
   [TransactionType.Withdrawal]: 1,
-  [TransactionType.Reward]: 2,
   [TransactionType.Payment]: 3,
   [TransactionType.PeerPullCredit]: 4,
   [TransactionType.PeerPullDebit]: 5,
@@ -228,12 +226,14 @@ export async function getTransactionById(
     case TransactionType.Withdrawal: {
       const withdrawalGroupId = parsedTx.withdrawalGroupId;
       return await wex.db.runReadWriteTx(
-        [
-          "withdrawalGroups",
-          "exchangeDetails",
-          "exchanges",
-          "operationRetries",
-        ],
+        {
+          storeNames: [
+            "withdrawalGroups",
+            "exchangeDetails",
+            "exchanges",
+            "operationRetries",
+          ],
+        },
         async (tx) => {
           const withdrawalGroupRecord =
             await tx.withdrawalGroups.get(withdrawalGroupId);
@@ -243,20 +243,22 @@ export async function getTransactionById(
           const opId = TaskIdentifiers.forWithdrawal(withdrawalGroupRecord);
           const ort = await tx.operationRetries.get(opId);
 
+          const exchangeDetails = await getExchangeWireDetailsInTx(
+            tx,
+            withdrawalGroupRecord.exchangeBaseUrl,
+          );
+          if (!exchangeDetails) throw Error("not exchange details");
+
           if (
             withdrawalGroupRecord.wgInfo.withdrawalType ===
             WithdrawalRecordType.BankIntegrated
           ) {
             return buildTransactionForBankIntegratedWithdraw(
               withdrawalGroupRecord,
+              exchangeDetails,
               ort,
             );
           }
-          const exchangeDetails = await getExchangeWireDetailsInTx(
-            tx,
-            withdrawalGroupRecord.exchangeBaseUrl,
-          );
-          if (!exchangeDetails) throw Error("not exchange details");
 
           return buildTransactionForManualWithdraw(
             withdrawalGroupRecord,
@@ -269,7 +271,7 @@ export async function getTransactionById(
 
     case TransactionType.DenomLoss: {
       const rec = await wex.db.runReadOnlyTx(
-        ["denomLossEvents"],
+        { storeNames: ["denomLossEvents"] },
         async (tx) => {
           return tx.denomLossEvents.get(parsedTx.denomLossEventId);
         },
@@ -286,13 +288,15 @@ export async function getTransactionById(
     case TransactionType.Payment: {
       const proposalId = parsedTx.proposalId;
       return await wex.db.runReadWriteTx(
-        [
-          "purchases",
-          "tombstones",
-          "operationRetries",
-          "contractTerms",
-          "refundGroups",
-        ],
+        {
+          storeNames: [
+            "purchases",
+            "tombstones",
+            "operationRetries",
+            "contractTerms",
+            "refundGroups",
+          ],
+        },
         async (tx) => {
           const purchase = await tx.purchases.get(proposalId);
           if (!purchase) throw Error("not found");
@@ -319,7 +323,7 @@ export async function getTransactionById(
       // FIXME: We should return info about the refresh here!;
       const refreshGroupId = parsedTx.refreshGroupId;
       return await wex.db.runReadOnlyTx(
-        ["refreshGroups", "operationRetries"],
+        { storeNames: ["refreshGroups", "operationRetries"] },
         async (tx) => {
           const refreshGroupRec = await tx.refreshGroups.get(refreshGroupId);
           if (!refreshGroupRec) {
@@ -333,14 +337,10 @@ export async function getTransactionById(
       );
     }
 
-    case TransactionType.Reward: {
-      throw Error("unsupported operation");
-    }
-
     case TransactionType.Deposit: {
       const depositGroupId = parsedTx.depositGroupId;
       return await wex.db.runReadWriteTx(
-        ["depositGroups", "operationRetries"],
+        { storeNames: ["depositGroups", "operationRetries"] },
         async (tx) => {
           const depositRecord = await tx.depositGroups.get(depositGroupId);
           if (!depositRecord) throw Error("not found");
@@ -355,7 +355,14 @@ export async function getTransactionById(
 
     case TransactionType.Refund: {
       return await wex.db.runReadOnlyTx(
-        ["refundGroups", "purchases", "operationRetries", "contractTerms"],
+        {
+          storeNames: [
+            "refundGroups",
+            "purchases",
+            "operationRetries",
+            "contractTerms",
+          ],
+        },
         async (tx) => {
           const refundRecord = await tx.refundGroups.get(
             parsedTx.refundGroupId,
@@ -373,7 +380,7 @@ export async function getTransactionById(
     }
     case TransactionType.PeerPullDebit: {
       return await wex.db.runReadWriteTx(
-        ["peerPullDebit", "contractTerms"],
+        { storeNames: ["peerPullDebit", "contractTerms"] },
         async (tx) => {
           const debit = await tx.peerPullDebit.get(parsedTx.peerPullDebitId);
           if (!debit) throw Error("not found");
@@ -392,12 +399,12 @@ export async function getTransactionById(
 
     case TransactionType.PeerPushDebit: {
       return await wex.db.runReadWriteTx(
-        ["peerPushDebit", "contractTerms"],
+        { storeNames: ["peerPushDebit", "contractTerms"] },
         async (tx) => {
           const debit = await tx.peerPushDebit.get(parsedTx.pursePub);
           if (!debit) throw Error("not found");
           const ct = await tx.contractTerms.get(debit.contractTermsHash);
-          checkDbInvariant(!!ct);
+          checkDbInvariant(!!ct, `no contract terms for p2p push ${parsedTx.pursePub}`);
           return buildTransactionForPushPaymentDebit(
             debit,
             ct.contractTermsRaw,
@@ -409,17 +416,19 @@ export async function getTransactionById(
     case TransactionType.PeerPushCredit: {
       const peerPushCreditId = parsedTx.peerPushCreditId;
       return await wex.db.runReadWriteTx(
-        [
-          "peerPushCredit",
-          "contractTerms",
-          "withdrawalGroups",
-          "operationRetries",
-        ],
+        {
+          storeNames: [
+            "peerPushCredit",
+            "contractTerms",
+            "withdrawalGroups",
+            "operationRetries",
+          ],
+        },
         async (tx) => {
           const pushInc = await tx.peerPushCredit.get(peerPushCreditId);
           if (!pushInc) throw Error("not found");
           const ct = await tx.contractTerms.get(pushInc.contractTermsHash);
-          checkDbInvariant(!!ct);
+          checkDbInvariant(!!ct, `no contract terms for p2p push ${peerPushCreditId}`);
 
           let wg: WithdrawalGroupRecord | undefined = undefined;
           let wgOrt: OperationRetryRecord | undefined = undefined;
@@ -431,7 +440,7 @@ export async function getTransactionById(
             }
           }
           const pushIncOpId = TaskIdentifiers.forPeerPushCredit(pushInc);
-          let pushIncOrt = await tx.operationRetries.get(pushIncOpId);
+          const pushIncOrt = await tx.operationRetries.get(pushIncOpId);
 
           return buildTransactionForPeerPushCredit(
             pushInc,
@@ -447,17 +456,19 @@ export async function getTransactionById(
     case TransactionType.PeerPullCredit: {
       const pursePub = parsedTx.pursePub;
       return await wex.db.runReadWriteTx(
-        [
-          "peerPullCredit",
-          "contractTerms",
-          "withdrawalGroups",
-          "operationRetries",
-        ],
+        {
+          storeNames: [
+            "peerPullCredit",
+            "contractTerms",
+            "withdrawalGroups",
+            "operationRetries",
+          ],
+        },
         async (tx) => {
           const pushInc = await tx.peerPullCredit.get(pursePub);
           if (!pushInc) throw Error("not found");
           const ct = await tx.contractTerms.get(pushInc.contractTermsHash);
-          checkDbInvariant(!!ct);
+          checkDbInvariant(!!ct, `no contract terms for p2p push ${pursePub}`);
 
           let wg: WithdrawalGroupRecord | undefined = undefined;
           let wgOrt: OperationRetryRecord | undefined = undefined;
@@ -580,6 +591,8 @@ function buildTransactionForPeerPullCredit(
         );
       });
     const txState = computePeerPullCreditTransactionState(pullCredit);
+    checkDbInvariant(wsr.instructedAmount !== undefined, "wg uninitialized");
+    checkDbInvariant(wsr.denomsSel !== undefined, "wg uninitialized");
     return {
       type: TransactionType.PeerPullCredit,
       txState,
@@ -645,13 +658,15 @@ function buildTransactionForPeerPushCredit(
   pushInc: PeerPushPaymentIncomingRecord,
   pushOrt: OperationRetryRecord | undefined,
   peerContractTerms: PeerContractTerms,
-  wsr: WithdrawalGroupRecord | undefined,
+  wg: WithdrawalGroupRecord | undefined,
   wsrOrt: OperationRetryRecord | undefined,
 ): Transaction {
-  if (wsr) {
-    if (wsr.wgInfo.withdrawalType !== WithdrawalRecordType.PeerPushCredit) {
+  if (wg) {
+    if (wg.wgInfo.withdrawalType !== WithdrawalRecordType.PeerPushCredit) {
       throw Error("invalid withdrawal group type for push payment credit");
     }
+    checkDbInvariant(wg.instructedAmount !== undefined, "wg uninitialized");
+    checkDbInvariant(wg.denomsSel !== undefined, "wg uninitialized");
 
     const txState = computePeerPushCreditTransactionState(pushInc);
     return {
@@ -659,15 +674,15 @@ function buildTransactionForPeerPushCredit(
       txState,
       txActions: computePeerPushCreditTransactionActions(pushInc),
       amountEffective: isUnsuccessfulTransaction(txState)
-        ? Amounts.stringify(Amounts.zeroOfAmount(wsr.instructedAmount))
-        : Amounts.stringify(wsr.denomsSel.totalCoinValue),
-      amountRaw: Amounts.stringify(wsr.instructedAmount),
-      exchangeBaseUrl: wsr.exchangeBaseUrl,
+        ? Amounts.stringify(Amounts.zeroOfAmount(wg.instructedAmount))
+        : Amounts.stringify(wg.denomsSel.totalCoinValue),
+      amountRaw: Amounts.stringify(wg.instructedAmount),
+      exchangeBaseUrl: wg.exchangeBaseUrl,
       info: {
         expiration: peerContractTerms.purse_expiration,
         summary: peerContractTerms.summary,
       },
-      timestamp: timestampPreciseFromDb(wsr.timestampStart),
+      timestamp: timestampPreciseFromDb(wg.timestampStart),
       transactionId: constructTransactionIdentifier({
         tag: TransactionType.PeerPushCredit,
         peerPushCreditId: pushInc.peerPushCreditId,
@@ -703,37 +718,44 @@ function buildTransactionForPeerPushCredit(
 }
 
 function buildTransactionForBankIntegratedWithdraw(
-  wgRecord: WithdrawalGroupRecord,
+  wg: WithdrawalGroupRecord,
+  exchangeDetails: ExchangeWireDetails,
   ort?: OperationRetryRecord,
 ): TransactionWithdrawal {
-  if (wgRecord.wgInfo.withdrawalType !== WithdrawalRecordType.BankIntegrated)
+  if (wg.wgInfo.withdrawalType !== WithdrawalRecordType.BankIntegrated) {
     throw Error("");
-
-  const txState = computeWithdrawalTransactionStatus(wgRecord);
+  }
+  const txState = computeWithdrawalTransactionStatus(wg);
+  const zero = Amounts.stringify(
+    Amounts.zeroOfCurrency(exchangeDetails.currency),
+  );
   return {
     type: TransactionType.Withdrawal,
     txState,
-    txActions: computeWithdrawalTransactionActions(wgRecord),
-    amountEffective: isUnsuccessfulTransaction(txState)
-      ? Amounts.stringify(Amounts.zeroOfAmount(wgRecord.instructedAmount))
-      : Amounts.stringify(wgRecord.denomsSel.totalCoinValue),
-    amountRaw: Amounts.stringify(wgRecord.instructedAmount),
+    txActions: computeWithdrawalTransactionActions(wg),
+    exchangeBaseUrl: wg.exchangeBaseUrl,
+    amountEffective:
+      isUnsuccessfulTransaction(txState) || !wg.denomsSel
+        ? zero
+        : Amounts.stringify(wg.denomsSel.totalCoinValue),
+    amountRaw: !wg.instructedAmount
+      ? zero
+      : Amounts.stringify(wg.instructedAmount),
     withdrawalDetails: {
       type: WithdrawalType.TalerBankIntegrationApi,
-      confirmed: wgRecord.wgInfo.bankInfo.timestampBankConfirmed ? true : false,
-      exchangeCreditAccountDetails: wgRecord.wgInfo.exchangeCreditAccounts,
-      reservePub: wgRecord.reservePub,
-      bankConfirmationUrl: wgRecord.wgInfo.bankInfo.confirmUrl,
+      confirmed: wg.wgInfo.bankInfo.timestampBankConfirmed ? true : false,
+      exchangeCreditAccountDetails: wg.wgInfo.exchangeCreditAccounts,
+      reservePub: wg.reservePub,
+      bankConfirmationUrl: wg.wgInfo.bankInfo.confirmUrl,
       reserveIsReady:
-        wgRecord.status === WithdrawalGroupStatus.Done ||
-        wgRecord.status === WithdrawalGroupStatus.PendingReady,
+        wg.status === WithdrawalGroupStatus.Done ||
+        wg.status === WithdrawalGroupStatus.PendingReady,
     },
-    kycUrl: wgRecord.kycUrl,
-    exchangeBaseUrl: wgRecord.exchangeBaseUrl,
-    timestamp: timestampPreciseFromDb(wgRecord.timestampStart),
+    kycUrl: wg.kycUrl,
+    timestamp: timestampPreciseFromDb(wg.timestampStart),
     transactionId: constructTransactionIdentifier({
       tag: TransactionType.Withdrawal,
-      withdrawalGroupId: wgRecord.withdrawalGroupId,
+      withdrawalGroupId: wg.withdrawalGroupId,
     }),
     ...(ort?.lastError ? { error: ort.lastError } : {}),
   };
@@ -750,50 +772,49 @@ export function isUnsuccessfulTransaction(state: TransactionState): boolean {
 }
 
 function buildTransactionForManualWithdraw(
-  withdrawalGroup: WithdrawalGroupRecord,
+  wg: WithdrawalGroupRecord,
   exchangeDetails: ExchangeWireDetails,
   ort?: OperationRetryRecord,
 ): TransactionWithdrawal {
-  if (withdrawalGroup.wgInfo.withdrawalType !== WithdrawalRecordType.BankManual)
+  if (wg.wgInfo.withdrawalType !== WithdrawalRecordType.BankManual)
     throw Error("");
 
   const plainPaytoUris =
     exchangeDetails.wireInfo?.accounts.map((x) => x.payto_uri) ?? [];
 
+  checkDbInvariant(wg.instructedAmount !== undefined, "wg uninitialized");
+  checkDbInvariant(wg.denomsSel !== undefined, "wg uninitialized");
   const exchangePaytoUris = augmentPaytoUrisForWithdrawal(
     plainPaytoUris,
-    withdrawalGroup.reservePub,
-    withdrawalGroup.instructedAmount,
+    wg.reservePub,
+    wg.instructedAmount,
   );
 
-  const txState = computeWithdrawalTransactionStatus(withdrawalGroup);
+  const txState = computeWithdrawalTransactionStatus(wg);
 
   return {
     type: TransactionType.Withdrawal,
     txState,
-    txActions: computeWithdrawalTransactionActions(withdrawalGroup),
+    txActions: computeWithdrawalTransactionActions(wg),
     amountEffective: isUnsuccessfulTransaction(txState)
-      ? Amounts.stringify(
-          Amounts.zeroOfAmount(withdrawalGroup.instructedAmount),
-        )
-      : Amounts.stringify(withdrawalGroup.denomsSel.totalCoinValue),
-    amountRaw: Amounts.stringify(withdrawalGroup.instructedAmount),
+      ? Amounts.stringify(Amounts.zeroOfAmount(wg.instructedAmount))
+      : Amounts.stringify(wg.denomsSel.totalCoinValue),
+    amountRaw: Amounts.stringify(wg.instructedAmount),
     withdrawalDetails: {
       type: WithdrawalType.ManualTransfer,
-      reservePub: withdrawalGroup.reservePub,
+      reservePub: wg.reservePub,
       exchangePaytoUris,
-      exchangeCreditAccountDetails:
-        withdrawalGroup.wgInfo.exchangeCreditAccounts,
+      exchangeCreditAccountDetails: wg.wgInfo.exchangeCreditAccounts,
       reserveIsReady:
-        withdrawalGroup.status === WithdrawalGroupStatus.Done ||
-        withdrawalGroup.status === WithdrawalGroupStatus.PendingReady,
+        wg.status === WithdrawalGroupStatus.Done ||
+        wg.status === WithdrawalGroupStatus.PendingReady,
     },
-    kycUrl: withdrawalGroup.kycUrl,
-    exchangeBaseUrl: withdrawalGroup.exchangeBaseUrl,
-    timestamp: timestampPreciseFromDb(withdrawalGroup.timestampStart),
+    kycUrl: wg.kycUrl,
+    exchangeBaseUrl: wg.exchangeBaseUrl,
+    timestamp: timestampPreciseFromDb(wg.timestampStart),
     transactionId: constructTransactionIdentifier({
       tag: TransactionType.Withdrawal,
-      withdrawalGroupId: withdrawalGroup.withdrawalGroupId,
+      withdrawalGroupId: wg.withdrawalGroupId,
     }),
     ...(ort?.lastError ? { error: ort.lastError } : {}),
   };
@@ -974,16 +995,22 @@ async function lookupMaybeContractData(
   return contractData;
 }
 
-async function buildTransactionForPurchase(
+function buildTransactionForPurchase(
   purchaseRecord: PurchaseRecord,
   contractData: WalletContractData,
   refundsInfo: RefundGroupRecord[],
   ort?: OperationRetryRecord,
-): Promise<Transaction> {
+): Transaction {
   const zero = Amounts.zeroOfAmount(contractData.amount);
 
   const info: OrderShortInfo = {
-    merchant: contractData.merchant,
+    merchant: {
+      name: contractData.merchant.name,
+      address: contractData.merchant.address,
+      email: contractData.merchant.email,
+      jurisdiction: contractData.merchant.jurisdiction,
+      website: contractData.merchant.website,
+    },
     orderId: contractData.orderId,
     summary: contractData.summary,
     summary_i18n: contractData.summaryI18n,
@@ -1007,8 +1034,8 @@ async function buildTransactionForPurchase(
   }));
 
   const timestamp = purchaseRecord.timestampAccept;
-  checkDbInvariant(!!timestamp);
-  checkDbInvariant(!!purchaseRecord.payInfo);
+  checkDbInvariant(!!timestamp, `purchase ${purchaseRecord.orderId} without accepted time`);
+  checkDbInvariant(!!purchaseRecord.payInfo, `purchase ${purchaseRecord.orderId} without payinfo`);
 
   const txState = computePayMerchantTransactionState(purchaseRecord);
   return {
@@ -1045,7 +1072,14 @@ export async function getWithdrawalTransactionByUri(
   request: WithdrawalTransactionByURIRequest,
 ): Promise<TransactionWithdrawal | undefined> {
   return await wex.db.runReadWriteTx(
-    ["withdrawalGroups", "exchangeDetails", "exchanges", "operationRetries"],
+    {
+      storeNames: [
+        "withdrawalGroups",
+        "exchangeDetails",
+        "exchanges",
+        "operationRetries",
+      ],
+    },
     async (tx) => {
       const withdrawalGroupRecord =
         await tx.withdrawalGroups.indexes.byTalerWithdrawUri.get(
@@ -1059,20 +1093,22 @@ export async function getWithdrawalTransactionByUri(
       const opId = TaskIdentifiers.forWithdrawal(withdrawalGroupRecord);
       const ort = await tx.operationRetries.get(opId);
 
+      const exchangeDetails = await getExchangeWireDetailsInTx(
+        tx,
+        withdrawalGroupRecord.exchangeBaseUrl,
+      );
+      if (!exchangeDetails) throw Error("not exchange details");
+
       if (
         withdrawalGroupRecord.wgInfo.withdrawalType ===
         WithdrawalRecordType.BankIntegrated
       ) {
         return buildTransactionForBankIntegratedWithdraw(
           withdrawalGroupRecord,
+          exchangeDetails,
           ort,
         );
       }
-      const exchangeDetails = await getExchangeWireDetailsInTx(
-        tx,
-        withdrawalGroupRecord.exchangeBaseUrl,
-      );
-      if (!exchangeDetails) throw Error("not exchange details");
 
       return buildTransactionForManualWithdraw(
         withdrawalGroupRecord,
@@ -1098,28 +1134,30 @@ export async function getTransactions(
   }
 
   await wex.db.runReadOnlyTx(
-    [
-      "coins",
-      "denominations",
-      "depositGroups",
-      "exchangeDetails",
-      "exchanges",
-      "operationRetries",
-      "peerPullDebit",
-      "peerPushDebit",
-      "peerPushCredit",
-      "peerPullCredit",
-      "planchets",
-      "purchases",
-      "contractTerms",
-      "recoupGroups",
-      "rewards",
-      "tombstones",
-      "withdrawalGroups",
-      "refreshGroups",
-      "refundGroups",
-      "denomLossEvents",
-    ],
+    {
+      storeNames: [
+        "coins",
+        "denominations",
+        "depositGroups",
+        "exchangeDetails",
+        "exchanges",
+        "operationRetries",
+        "peerPullDebit",
+        "peerPushDebit",
+        "peerPushCredit",
+        "peerPullCredit",
+        "planchets",
+        "purchases",
+        "contractTerms",
+        "recoupGroups",
+        "rewards",
+        "tombstones",
+        "withdrawalGroups",
+        "refreshGroups",
+        "refundGroups",
+        "denomLossEvents",
+      ],
+    },
     async (tx) => {
       await iterRecordsForPeerPushDebit(tx, filter, async (pi) => {
         const amount = Amounts.parseOrThrow(pi.amount);
@@ -1137,7 +1175,7 @@ export async function getTransactions(
           return;
         }
         const ct = await tx.contractTerms.get(pi.contractTermsHash);
-        checkDbInvariant(!!ct);
+        checkDbInvariant(!!ct, `no contract terms for p2p push ${pi.pursePub}`);
         transactions.push(
           buildTransactionForPushPaymentDebit(pi, ct.contractTermsRaw),
         );
@@ -1211,9 +1249,9 @@ export async function getTransactions(
           }
         }
         const pushIncOpId = TaskIdentifiers.forPeerPushCredit(pi);
-        let pushIncOrt = await tx.operationRetries.get(pushIncOpId);
+        const pushIncOrt = await tx.operationRetries.get(pushIncOpId);
 
-        checkDbInvariant(!!ct);
+        checkDbInvariant(!!ct, `no contract terms for p2p push ${pi.pursePub}`);
         transactions.push(
           buildTransactionForPeerPushCredit(
             pi,
@@ -1245,9 +1283,9 @@ export async function getTransactions(
           }
         }
         const pushIncOpId = TaskIdentifiers.forPeerPullPaymentInitiation(pi);
-        let pushIncOrt = await tx.operationRetries.get(pushIncOpId);
+        const pushIncOrt = await tx.operationRetries.get(pushIncOpId);
 
-        checkDbInvariant(!!ct);
+        checkDbInvariant(!!ct, `no contract terms for p2p push ${pi.pursePub}`);
         transactions.push(
           buildTransactionForPeerPullCredit(
             pi,
@@ -1313,6 +1351,13 @@ export async function getTransactions(
       });
 
       await iterRecordsForWithdrawal(tx, filter, async (wsr) => {
+        if (
+          wsr.rawWithdrawalAmount === undefined ||
+          wsr.exchangeBaseUrl == undefined
+        ) {
+          // skip prepared withdrawals which has not been confirmed
+          return;
+        }
         const exchangesInTx = [wsr.exchangeBaseUrl];
         if (
           shouldSkipCurrency(
@@ -1342,12 +1387,7 @@ export async function getTransactions(
             // FIXME: If this is an orphan withdrawal, still report it as a withdrawal!
             // FIXME: Still report if requested with verbose option?
             return;
-          case WithdrawalRecordType.BankIntegrated:
-            transactions.push(
-              buildTransactionForBankIntegratedWithdraw(wsr, ort),
-            );
-            return;
-          case WithdrawalRecordType.BankManual: {
+          case WithdrawalRecordType.BankIntegrated: {
             const exchangeDetails = await getExchangeWireDetailsInTx(
               tx,
               wsr.exchangeBaseUrl,
@@ -1357,6 +1397,25 @@ export async function getTransactions(
               return;
             }
 
+            transactions.push(
+              buildTransactionForBankIntegratedWithdraw(
+                wsr,
+                exchangeDetails,
+                ort,
+              ),
+            );
+            return;
+          }
+
+          case WithdrawalRecordType.BankManual: {
+            const exchangeDetails = await getExchangeWireDetailsInTx(
+              tx,
+              wsr.exchangeBaseUrl,
+            );
+            if (!exchangeDetails) {
+              // FIXME: report somehow
+              return;
+            }
             transactions.push(
               buildTransactionForManualWithdraw(wsr, exchangeDetails, ort),
             );
@@ -1457,7 +1516,7 @@ export async function getTransactions(
         );
 
         transactions.push(
-          await buildTransactionForPurchase(
+          buildTransactionForPurchase(
             purchase,
             contractData,
             refunds,
@@ -1531,7 +1590,6 @@ export type ParsedTransactionIdentifier =
   | { tag: TransactionType.PeerPushDebit; pursePub: string }
   | { tag: TransactionType.Refresh; refreshGroupId: string }
   | { tag: TransactionType.Refund; refundGroupId: string }
-  | { tag: TransactionType.Reward; walletRewardId: string }
   | { tag: TransactionType.Withdrawal; withdrawalGroupId: string }
   | { tag: TransactionType.InternalWithdrawal; withdrawalGroupId: string }
   | { tag: TransactionType.Recoup; recoupGroupId: string }
@@ -1557,8 +1615,6 @@ export function constructTransactionIdentifier(
       return `txn:${pTxId.tag}:${pTxId.refreshGroupId}` as TransactionIdStr;
     case TransactionType.Refund:
       return `txn:${pTxId.tag}:${pTxId.refundGroupId}` as TransactionIdStr;
-    case TransactionType.Reward:
-      return `txn:${pTxId.tag}:${pTxId.walletRewardId}` as TransactionIdStr;
     case TransactionType.Withdrawal:
       return `txn:${pTxId.tag}:${pTxId.withdrawalGroupId}` as TransactionIdStr;
     case TransactionType.InternalWithdrawal:
@@ -1616,11 +1672,6 @@ export function parseTransactionIdentifier(
         tag: TransactionType.Refund,
         refundGroupId: rest[0],
       };
-    case TransactionType.Reward:
-      return {
-        tag: TransactionType.Reward,
-        walletRewardId: rest[0],
-      };
     case TransactionType.Withdrawal:
       return {
         tag: TransactionType.Withdrawal,
@@ -1669,11 +1720,6 @@ function maybeTaskFromTransaction(
         tag: PendingTaskType.Purchase,
         proposalId: parsedTx.proposalId,
       });
-    case TransactionType.Reward:
-      return constructTaskIdentifier({
-        tag: PendingTaskType.RewardPickup,
-        walletRewardId: parsedTx.walletRewardId,
-      });
     case TransactionType.Refresh:
       return constructTaskIdentifier({
         tag: PendingTaskType.Refresh,
@@ -1721,7 +1767,18 @@ export async function retryTransaction(
   logger.info(`resetting retry timeout for ${transactionId}`);
   const taskId = maybeTaskFromTransaction(transactionId);
   if (taskId) {
-    wex.taskScheduler.resetTaskRetries(taskId);
+    await wex.taskScheduler.resetTaskRetries(taskId);
+  }
+}
+
+/**
+ * Reset the task retry counter for all tasks.
+ */
+export async function retryAll(wex: WalletExecutionContext): Promise<void> {
+  await wex.taskScheduler.ensureRunning();
+  const tasks = wex.taskScheduler.getActiveTasks();
+  for (const task of tasks) {
+    await wex.taskScheduler.resetTaskRetries(task);
   }
 }
 
@@ -1753,8 +1810,6 @@ async function getContextForTransaction(
       return new PeerPushCreditTransactionContext(wex, tx.peerPushCreditId);
     case TransactionType.Refund:
       return new RefundTransactionContext(wex, tx.refundGroupId);
-    case TransactionType.Reward:
-      return new RewardTransactionContext(wex, tx.walletRewardId);
     case TransactionType.Recoup:
       //return new RecoupTransactionContext(ws, tx.recoupGroupId);
       throw new Error("not yet supported");
