@@ -22,7 +22,7 @@
 /**
  * Imports.
  */
-import { IDBFactory } from "@gnu-taler/idb-bridge";
+import { IDBDatabase, IDBFactory } from "@gnu-taler/idb-bridge";
 import {
   AbsoluteTime,
   ActiveTask,
@@ -55,7 +55,6 @@ import {
   PrepareWithdrawExchangeRequest,
   PrepareWithdrawExchangeResponse,
   RecoverStoredBackupRequest,
-  RetryLoopOpts,
   StoredBackupList,
   TalerError,
   TalerErrorCode,
@@ -71,6 +70,7 @@ import {
   WalletCoreVersion,
   WalletNotification,
   WalletRunConfig,
+  canonicalizeBaseUrl,
   checkDbInvariant,
   codecForAbortTransaction,
   codecForAcceptBankIntegratedWithdrawalRequest,
@@ -83,10 +83,13 @@ import {
   codecForAddKnownBankAccounts,
   codecForAny,
   codecForApplyDevExperiment,
+  codecForCanonicalizeBaseUrlRequest,
+  codecForCheckPayTemplateRequest,
   codecForCheckPeerPullPaymentRequest,
   codecForCheckPeerPushDebitRequest,
   codecForConfirmPayRequest,
   codecForConfirmPeerPushPaymentRequest,
+  codecForConfirmWithdrawalRequestRequest,
   codecForConvertAmountRequest,
   codecForCreateDepositGroupRequest,
   codecForDeleteExchangeRequest,
@@ -104,6 +107,7 @@ import {
   codecForGetExchangeTosRequest,
   codecForGetWithdrawalDetailsForAmountRequest,
   codecForGetWithdrawalDetailsForUri,
+  codecForHintNetworkAvailabilityRequest,
   codecForImportDbRequest,
   codecForInitRequest,
   codecForInitiatePeerPullPaymentRequest,
@@ -112,6 +116,7 @@ import {
   codecForIntegrationTestV2Args,
   codecForListExchangesForScopedCurrencyRequest,
   codecForListKnownBankAccounts,
+  codecForPrepareBankIntegratedWithdrawalRequest,
   codecForPrepareDepositRequest,
   codecForPreparePayRequest,
   codecForPreparePayTemplateRequest,
@@ -131,6 +136,7 @@ import {
   codecForSuspendTransaction,
   codecForTestPayArgs,
   codecForTestingGetDenomStatsRequest,
+  codecForTestingGetReserveHistoryRequest,
   codecForTestingListTasksForTransactionRequest,
   codecForTestingSetTimetravelRequest,
   codecForTransactionByIdRequest,
@@ -146,11 +152,15 @@ import {
   parsePaytoUri,
   parseTalerUri,
   performanceNow,
+  safeStringifyException,
   sampleWalletCoreTransactions,
   setDangerousTimetravel,
   validateIban,
 } from "@gnu-taler/taler-util";
-import type { HttpRequestLibrary } from "@gnu-taler/taler-util/http";
+import {
+  readSuccessResponseJsonOrThrow,
+  type HttpRequestLibrary,
+} from "@gnu-taler/taler-util/http";
 import {
   getUserAttentions,
   getUserAttentionsUnreadCount,
@@ -220,6 +230,7 @@ import {
   observeTalerCrypto,
 } from "./observable-wrappers.js";
 import {
+  checkPayForTemplate,
   confirmPay,
   getContractTermsDetails,
   preparePayForTemplate,
@@ -244,18 +255,25 @@ import {
   checkPeerPushDebit,
   initiatePeerPushDebit,
 } from "./pay-peer-push-debit.js";
-import { DbAccess } from "./query.js";
+import {
+  AfterCommitInfo,
+  DbAccess,
+  DbAccessImpl,
+  TriggerSpec,
+} from "./query.js";
 import { forceRefresh } from "./refresh.js";
 import {
   TaskScheduler,
   TaskSchedulerImpl,
   convertTaskToTransactionId,
+  getActiveTaskIds,
   listTaskForTransactionId,
 } from "./shepherd.js";
 import {
   runIntegrationTest,
   runIntegrationTest2,
   testPay,
+  waitTasksDone,
   waitTransactionState,
   waitUntilAllTransactionsFinal,
   waitUntilRefreshesDone,
@@ -271,6 +289,7 @@ import {
   getWithdrawalTransactionByUri,
   parseTransactionIdentifier,
   resumeTransaction,
+  retryAll,
   retryTransaction,
   suspendTransaction,
 } from "./transactions.js";
@@ -289,9 +308,11 @@ import {
 } from "./wallet-api-types.js";
 import {
   acceptWithdrawalFromUri,
+  confirmWithdrawal,
   createManualWithdrawal,
   getWithdrawalDetailsForAmount,
   getWithdrawalDetailsForUri,
+  prepareBankIntegratedWithdrawal,
 } from "./withdraw.js";
 
 const logger = new Logger("wallet.ts");
@@ -326,28 +347,31 @@ type CancelFn = () => void;
  */
 async function fillDefaults(wex: WalletExecutionContext): Promise<void> {
   const notifications: WalletNotification[] = [];
-  await wex.db.runReadWriteTx(["config", "exchanges"], async (tx) => {
-    const appliedRec = await tx.config.get("currencyDefaultsApplied");
-    let alreadyApplied = appliedRec ? !!appliedRec.value : false;
-    if (alreadyApplied) {
-      logger.trace("defaults already applied");
-      return;
-    }
-    for (const exch of wex.ws.config.builtin.exchanges) {
-      const resp = await addPresetExchangeEntry(
-        tx,
-        exch.exchangeBaseUrl,
-        exch.currencyHint,
-      );
-      if (resp.notification) {
-        notifications.push(resp.notification);
+  await wex.db.runReadWriteTx(
+    { storeNames: ["config", "exchanges"] },
+    async (tx) => {
+      const appliedRec = await tx.config.get("currencyDefaultsApplied");
+      let alreadyApplied = appliedRec ? !!appliedRec.value : false;
+      if (alreadyApplied) {
+        logger.trace("defaults already applied");
+        return;
       }
-    }
-    await tx.config.put({
-      key: ConfigRecordKey.CurrencyDefaultsApplied,
-      value: true,
-    });
-  });
+      for (const exch of wex.ws.config.builtin.exchanges) {
+        const resp = await addPresetExchangeEntry(
+          tx,
+          exch.exchangeBaseUrl,
+          exch.currencyHint,
+        );
+        if (resp.notification) {
+          notifications.push(resp.notification);
+        }
+      }
+      await tx.config.put({
+        key: ConfigRecordKey.CurrencyDefaultsApplied,
+        value: true,
+      });
+    },
+  );
   for (const notif of notifications) {
     wex.ws.notify(notif);
   }
@@ -382,7 +406,7 @@ async function listKnownBankAccounts(
   currency?: string,
 ): Promise<KnownBankAccounts> {
   const accounts: KnownBankAccountsInfo[] = [];
-  await wex.db.runReadOnlyTx(["bankAccounts"], async (tx) => {
+  await wex.db.runReadOnlyTx({ storeNames: ["bankAccounts"] }, async (tx) => {
     const knownAccounts = await tx.bankAccounts.iter().toArray();
     for (const r of knownAccounts) {
       if (currency && currency !== r.currency) {
@@ -410,7 +434,7 @@ async function addKnownBankAccounts(
   alias: string,
   currency: string,
 ): Promise<void> {
-  await wex.db.runReadWriteTx(["bankAccounts"], async (tx) => {
+  await wex.db.runReadWriteTx({ storeNames: ["bankAccounts"] }, async (tx) => {
     tx.bankAccounts.put({
       uri: payto,
       alias: alias,
@@ -427,7 +451,7 @@ async function forgetKnownBankAccounts(
   wex: WalletExecutionContext,
   payto: string,
 ): Promise<void> {
-  await wex.db.runReadWriteTx(["bankAccounts"], async (tx) => {
+  await wex.db.runReadWriteTx({ storeNames: ["bankAccounts"] }, async (tx) => {
     const account = await tx.bankAccounts.get(payto);
     if (!account) {
       throw Error(`account not found: ${payto}`);
@@ -442,39 +466,42 @@ async function setCoinSuspended(
   coinPub: string,
   suspended: boolean,
 ): Promise<void> {
-  await wex.db.runReadWriteTx(["coins", "coinAvailability"], async (tx) => {
-    const c = await tx.coins.get(coinPub);
-    if (!c) {
-      logger.warn(`coin ${coinPub} not found, won't suspend`);
-      return;
-    }
-    const coinAvailability = await tx.coinAvailability.get([
-      c.exchangeBaseUrl,
-      c.denomPubHash,
-      c.maxAge,
-    ]);
-    checkDbInvariant(!!coinAvailability);
-    if (suspended) {
-      if (c.status !== CoinStatus.Fresh) {
+  await wex.db.runReadWriteTx(
+    { storeNames: ["coins", "coinAvailability"] },
+    async (tx) => {
+      const c = await tx.coins.get(coinPub);
+      if (!c) {
+        logger.warn(`coin ${coinPub} not found, won't suspend`);
         return;
       }
-      if (coinAvailability.freshCoinCount === 0) {
-        throw Error(
-          `invalid coin count ${coinAvailability.freshCoinCount} in DB`,
-        );
+      const coinAvailability = await tx.coinAvailability.get([
+        c.exchangeBaseUrl,
+        c.denomPubHash,
+        c.maxAge,
+      ]);
+      checkDbInvariant(!!coinAvailability, `no denom info for ${c.denomPubHash} age ${c.maxAge}`);
+      if (suspended) {
+        if (c.status !== CoinStatus.Fresh) {
+          return;
+        }
+        if (coinAvailability.freshCoinCount === 0) {
+          throw Error(
+            `invalid coin count ${coinAvailability.freshCoinCount} in DB`,
+          );
+        }
+        coinAvailability.freshCoinCount--;
+        c.status = CoinStatus.FreshSuspended;
+      } else {
+        if (c.status == CoinStatus.Dormant) {
+          return;
+        }
+        coinAvailability.freshCoinCount++;
+        c.status = CoinStatus.Fresh;
       }
-      coinAvailability.freshCoinCount--;
-      c.status = CoinStatus.FreshSuspended;
-    } else {
-      if (c.status == CoinStatus.Dormant) {
-        return;
-      }
-      coinAvailability.freshCoinCount++;
-      c.status = CoinStatus.Fresh;
-    }
-    await tx.coins.put(c);
-    await tx.coinAvailability.put(coinAvailability);
-  });
+      await tx.coins.put(c);
+      await tx.coinAvailability.put(coinAvailability);
+    },
+  );
 }
 
 /**
@@ -483,55 +510,58 @@ async function setCoinSuspended(
 async function dumpCoins(wex: WalletExecutionContext): Promise<CoinDumpJson> {
   const coinsJson: CoinDumpJson = { coins: [] };
   logger.info("dumping coins");
-  await wex.db.runReadOnlyTx(["coins", "denominations"], async (tx) => {
-    const coins = await tx.coins.iter().toArray();
-    for (const c of coins) {
-      const denom = await tx.denominations.get([
-        c.exchangeBaseUrl,
-        c.denomPubHash,
-      ]);
-      if (!denom) {
-        logger.warn("no denom found for coin");
-        continue;
+  await wex.db.runReadOnlyTx(
+    { storeNames: ["coins", "denominations"] },
+    async (tx) => {
+      const coins = await tx.coins.iter().toArray();
+      for (const c of coins) {
+        const denom = await tx.denominations.get([
+          c.exchangeBaseUrl,
+          c.denomPubHash,
+        ]);
+        if (!denom) {
+          logger.warn("no denom found for coin");
+          continue;
+        }
+        const cs = c.coinSource;
+        let refreshParentCoinPub: string | undefined;
+        if (cs.type == CoinSourceType.Refresh) {
+          refreshParentCoinPub = cs.oldCoinPub;
+        }
+        let withdrawalReservePub: string | undefined;
+        if (cs.type == CoinSourceType.Withdraw) {
+          withdrawalReservePub = cs.reservePub;
+        }
+        const denomInfo = await getDenomInfo(
+          wex,
+          tx,
+          c.exchangeBaseUrl,
+          c.denomPubHash,
+        );
+        if (!denomInfo) {
+          logger.warn("no denomination found for coin");
+          continue;
+        }
+        coinsJson.coins.push({
+          coin_pub: c.coinPub,
+          denom_pub: denomInfo.denomPub,
+          denom_pub_hash: c.denomPubHash,
+          denom_value: denom.value,
+          exchange_base_url: c.exchangeBaseUrl,
+          refresh_parent_coin_pub: refreshParentCoinPub,
+          withdrawal_reserve_pub: withdrawalReservePub,
+          coin_status: c.status,
+          ageCommitmentProof: c.ageCommitmentProof,
+          spend_allocation: c.spendAllocation
+            ? {
+                amount: c.spendAllocation.amount,
+                id: c.spendAllocation.id,
+              }
+            : undefined,
+        });
       }
-      const cs = c.coinSource;
-      let refreshParentCoinPub: string | undefined;
-      if (cs.type == CoinSourceType.Refresh) {
-        refreshParentCoinPub = cs.oldCoinPub;
-      }
-      let withdrawalReservePub: string | undefined;
-      if (cs.type == CoinSourceType.Withdraw) {
-        withdrawalReservePub = cs.reservePub;
-      }
-      const denomInfo = await getDenomInfo(
-        wex,
-        tx,
-        c.exchangeBaseUrl,
-        c.denomPubHash,
-      );
-      if (!denomInfo) {
-        logger.warn("no denomination found for coin");
-        continue;
-      }
-      coinsJson.coins.push({
-        coin_pub: c.coinPub,
-        denom_pub: denomInfo.denomPub,
-        denom_pub_hash: c.denomPubHash,
-        denom_value: denom.value,
-        exchange_base_url: c.exchangeBaseUrl,
-        refresh_parent_coin_pub: refreshParentCoinPub,
-        withdrawal_reserve_pub: withdrawalReservePub,
-        coin_status: c.status,
-        ageCommitmentProof: c.ageCommitmentProof,
-        spend_allocation: c.spendAllocation
-          ? {
-              amount: c.spendAllocation.amount,
-              id: c.spendAllocation.id,
-            }
-          : undefined,
-      });
-    }
-  });
+    },
+  );
   return coinsJson;
 }
 
@@ -635,9 +665,6 @@ async function handlePrepareWithdrawExchange(
   }
   const exchangeBaseUrl = parsedUri.exchangeBaseUrl;
   const exchange = await fetchFreshExchange(wex, exchangeBaseUrl);
-  if (parsedUri.exchangePub && exchange.masterPub != parsedUri.exchangePub) {
-    throw Error("mismatch of exchange master public key (URI vs actual)");
-  }
   if (parsedUri.amount) {
     const amt = Amounts.parseOrThrow(parsedUri.amount);
     if (amt.currency !== exchange.currency) {
@@ -665,7 +692,7 @@ export interface PendingOperationsResponse {
 /**
  * Implementation of the "wallet-core" API.
  */
-async function dispatchRequestInternal<Op extends WalletApiOperation>(
+async function dispatchRequestInternal(
   wex: WalletExecutionContext,
   cts: CancellationToken.Source,
   operation: WalletApiOperation,
@@ -697,18 +724,17 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
     case WalletApiOperation.InitWallet: {
       const req = codecForInitRequest().decode(payload);
 
-      logger.info(`init request: ${j2s(req)}`);
-
-      if (wex.ws.initCalled) {
-        logger.info("initializing wallet (repeat initialization)");
-      } else {
-        logger.info("initializing wallet (first initialization)");
+      if (logger.shouldLogTrace()) {
+        const initType = wex.ws.initCalled
+          ? "repeat initialization"
+          : "first initialization";
+        logger.trace(`init request (${initType}): ${j2s(req)}`);
       }
 
       // Write to the DB to make sure that we're failing early in
       // case the DB is not writeable.
       try {
-        await wex.db.runReadWriteTx(["config"], async (tx) => {
+        await wex.db.runReadWriteTx({ storeNames: ["config"] }, async (tx) => {
           tx.config.put({
             key: ConfigRecordKey.LastInitInfo,
             value: timestampProtocolToDb(TalerProtocolTimestamp.now()),
@@ -720,10 +746,8 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
           innerError: getErrorDetailFromException(e),
         });
       }
-
       wex.ws.initWithConfig(applyRunConfigDefaults(req.config));
 
-      wex.ws.initCalled = true;
       if (wex.ws.config.testing.skipDefaults) {
         logger.trace("skipping defaults");
       } else {
@@ -733,6 +757,14 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       const resp: InitResponse = {
         versionInfo: getVersion(wex),
       };
+
+      if (req.config?.lazyTaskLoop) {
+        logger.trace("lazily starting task loop");
+      } else {
+        await wex.taskScheduler.ensureRunning();
+      }
+
+      wex.ws.initCalled = true;
       return resp;
     }
     case WalletApiOperation.WithdrawTestkudos: {
@@ -793,9 +825,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
     }
     case WalletApiOperation.AddExchange: {
       const req = codecForAddExchangeRequest().decode(payload);
-      await fetchFreshExchange(wex, req.exchangeBaseUrl, {
-        expectedMasterPub: req.masterPub,
-      });
+      await fetchFreshExchange(wex, req.exchangeBaseUrl, {});
       return {};
     }
     case WalletApiOperation.TestingPing: {
@@ -815,20 +845,24 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
         numLost: 0,
         numOffered: 0,
       };
-      await wex.db.runReadOnlyTx(["denominations"], async (tx) => {
-        const denoms = await tx.denominations.indexes.byExchangeBaseUrl.getAll(
-          req.exchangeBaseUrl,
-        );
-        for (const d of denoms) {
-          denomStats.numKnown++;
-          if (d.isOffered) {
-            denomStats.numOffered++;
+      await wex.db.runReadOnlyTx(
+        { storeNames: ["denominations"] },
+        async (tx) => {
+          const denoms =
+            await tx.denominations.indexes.byExchangeBaseUrl.getAll(
+              req.exchangeBaseUrl,
+            );
+          for (const d of denoms) {
+            denomStats.numKnown++;
+            if (d.isOffered) {
+              denomStats.numOffered++;
+            }
+            if (d.isLost) {
+              denomStats.numLost++;
+            }
           }
-          if (d.isLost) {
-            denomStats.numLost++;
-          }
-        }
-      });
+        },
+      );
       return denomStats;
     }
     case WalletApiOperation.ListExchanges: {
@@ -876,10 +910,36 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
     }
     case WalletApiOperation.GetWithdrawalDetailsForUri: {
       const req = codecForGetWithdrawalDetailsForUri().decode(payload);
-      return await getWithdrawalDetailsForUri(wex, req.talerWithdrawUri, {
-        notifyChangeFromPendingTimeoutMs: req.notifyChangeFromPendingTimeoutMs,
-        restrictAge: req.restrictAge,
+      return await getWithdrawalDetailsForUri(wex, req.talerWithdrawUri);
+    }
+    case WalletApiOperation.TestingGetReserveHistory: {
+      const req = codecForTestingGetReserveHistoryRequest().decode(payload);
+      const reserve = await wex.db.runReadOnlyTx(
+        { storeNames: ["reserves"] },
+        async (tx) => {
+          return tx.reserves.indexes.byReservePub.get(req.reservePub);
+        },
+      );
+      if (!reserve) {
+        throw Error("no reserve pub found");
+      }
+      const sigResp = await wex.cryptoApi.signReserveHistoryReq({
+        reservePriv: reserve.reservePriv,
+        startOffset: 0,
       });
+      const exchangeBaseUrl = req.exchangeBaseUrl;
+      const url = new URL(
+        `reserves/${req.reservePub}/history`,
+        exchangeBaseUrl,
+      );
+      const resp = await wex.http.fetch(url.href, {
+        headers: { ["Taler-Reserve-History-Signature"]: sigResp.sig },
+      });
+      const historyJson = await readSuccessResponseJsonOrThrow(
+        resp,
+        codecForAny(),
+      );
+      return historyJson;
     }
     case WalletApiOperation.AcceptManualWithdrawal: {
       const req = codecForAcceptManualWithdrawalRequest().decode(payload);
@@ -887,6 +947,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
         amount: Amounts.parseOrThrow(req.amount),
         exchangeBaseUrl: req.exchangeBaseUrl,
         restrictAge: req.restrictAge,
+        forceReservePriv: req.forceReservePriv,
       });
       return res;
     }
@@ -939,6 +1000,19 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
         talerWithdrawUri: req.talerWithdrawUri,
         forcedDenomSel: req.forcedDenomSel,
         restrictAge: req.restrictAge,
+        amount: req.amount,
+      });
+    }
+    case WalletApiOperation.ConfirmWithdrawal: {
+      const req = codecForConfirmWithdrawalRequestRequest().decode(payload);
+      return confirmWithdrawal(wex, req);
+    }
+    case WalletApiOperation.PrepareBankIntegratedWithdrawal: {
+      const req =
+        codecForPrepareBankIntegratedWithdrawalRequest().decode(payload);
+      return prepareBankIntegratedWithdrawal(wex, {
+        talerWithdrawUri: req.talerWithdrawUri,
+        selectedExchange: req.selectedExchange,
       });
     }
     case WalletApiOperation.GetExchangeTos: {
@@ -977,6 +1051,10 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       const req = codecForPrepareWithdrawExchangeRequest().decode(payload);
       return handlePrepareWithdrawExchange(wex, req);
     }
+    case WalletApiOperation.CheckPayForTemplate: {
+      const req = codecForCheckPayTemplateRequest().decode(payload);
+      return await checkPayForTemplate(wex, req);
+    }
     case WalletApiOperation.PreparePayForUri: {
       const req = codecForPreparePayRequest().decode(payload);
       return await preparePayForUri(wex, req.talerPayUri);
@@ -1012,12 +1090,12 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       return {};
     }
     case WalletApiOperation.GetActiveTasks: {
-      const allTasksId = wex.taskScheduler.getActiveTasks();
+      const allTasksId = (await getActiveTaskIds(wex.ws)).taskIds;
 
       const tasksInfo = await Promise.all(
         allTasksId.map(async (id) => {
-          return await wex.ws.db.runReadOnlyTx(
-            ["operationRetries"],
+          return await wex.db.runReadOnlyTx(
+            { storeNames: ["operationRetries"] },
             async (tx) => {
               return tx.operationRetries.get(id);
             },
@@ -1039,8 +1117,8 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
         const lastError = d?.lastError;
 
         return {
-          id: taskId,
-          counter,
+          taskId: taskId,
+          retryCounter: counter,
           firstTry,
           nextTry,
           lastError,
@@ -1161,10 +1239,16 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       await loadBackupRecovery(wex, req);
       return {};
     }
-    // case WalletApiOperation.GetPlanForOperation: {
-    //   const req = codecForGetPlanForOperationRequest().decode(payload);
-    //   return await getPlanForOperation(ws, req);
-    // }
+    case WalletApiOperation.HintNetworkAvailability: {
+      const req = codecForHintNetworkAvailabilityRequest().decode(payload);
+      if (req.isNetworkAvailable) {
+        await retryAll(wex);
+      } else {
+        // We're not doing anything right now, but we could stop showing
+        // certain errors!
+      }
+      return {};
+    }
     case WalletApiOperation.ConvertDepositAmount: {
       const req = codecForConvertAmountRequest.decode(payload);
       return await convertDepositAmount(wex, req);
@@ -1236,106 +1320,139 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       const resp: ListGlobalCurrencyExchangesResponse = {
         exchanges: [],
       };
-      await wex.db.runReadOnlyTx(["globalCurrencyExchanges"], async (tx) => {
-        const gceList = await tx.globalCurrencyExchanges.iter().toArray();
-        for (const gce of gceList) {
-          resp.exchanges.push({
-            currency: gce.currency,
-            exchangeBaseUrl: gce.exchangeBaseUrl,
-            exchangeMasterPub: gce.exchangeMasterPub,
-          });
-        }
-      });
+      await wex.db.runReadOnlyTx(
+        { storeNames: ["globalCurrencyExchanges"] },
+        async (tx) => {
+          const gceList = await tx.globalCurrencyExchanges.iter().toArray();
+          for (const gce of gceList) {
+            resp.exchanges.push({
+              currency: gce.currency,
+              exchangeBaseUrl: gce.exchangeBaseUrl,
+              exchangeMasterPub: gce.exchangeMasterPub,
+            });
+          }
+        },
+      );
       return resp;
     }
     case WalletApiOperation.ListGlobalCurrencyAuditors: {
       const resp: ListGlobalCurrencyAuditorsResponse = {
         auditors: [],
       };
-      await wex.db.runReadOnlyTx(["globalCurrencyAuditors"], async (tx) => {
-        const gcaList = await tx.globalCurrencyAuditors.iter().toArray();
-        for (const gca of gcaList) {
-          resp.auditors.push({
-            currency: gca.currency,
-            auditorBaseUrl: gca.auditorBaseUrl,
-            auditorPub: gca.auditorPub,
-          });
-        }
-      });
+      await wex.db.runReadOnlyTx(
+        { storeNames: ["globalCurrencyAuditors"] },
+        async (tx) => {
+          const gcaList = await tx.globalCurrencyAuditors.iter().toArray();
+          for (const gca of gcaList) {
+            resp.auditors.push({
+              currency: gca.currency,
+              auditorBaseUrl: gca.auditorBaseUrl,
+              auditorPub: gca.auditorPub,
+            });
+          }
+        },
+      );
       return resp;
     }
     case WalletApiOperation.AddGlobalCurrencyExchange: {
       const req = codecForAddGlobalCurrencyExchangeRequest().decode(payload);
-      await wex.db.runReadWriteTx(["globalCurrencyExchanges"], async (tx) => {
-        const key = [req.currency, req.exchangeBaseUrl, req.exchangeMasterPub];
-        const existingRec =
-          await tx.globalCurrencyExchanges.indexes.byCurrencyAndUrlAndPub.get(
-            key,
-          );
-        if (existingRec) {
-          return;
-        }
-        wex.ws.exchangeCache.clear();
-        await tx.globalCurrencyExchanges.add({
-          currency: req.currency,
-          exchangeBaseUrl: req.exchangeBaseUrl,
-          exchangeMasterPub: req.exchangeMasterPub,
-        });
-      });
+      await wex.db.runReadWriteTx(
+        { storeNames: ["globalCurrencyExchanges"] },
+        async (tx) => {
+          const key = [
+            req.currency,
+            req.exchangeBaseUrl,
+            req.exchangeMasterPub,
+          ];
+          const existingRec =
+            await tx.globalCurrencyExchanges.indexes.byCurrencyAndUrlAndPub.get(
+              key,
+            );
+          if (existingRec) {
+            return;
+          }
+          wex.ws.exchangeCache.clear();
+          await tx.globalCurrencyExchanges.add({
+            currency: req.currency,
+            exchangeBaseUrl: req.exchangeBaseUrl,
+            exchangeMasterPub: req.exchangeMasterPub,
+          });
+        },
+      );
       return {};
     }
     case WalletApiOperation.RemoveGlobalCurrencyExchange: {
       const req = codecForRemoveGlobalCurrencyExchangeRequest().decode(payload);
-      await wex.db.runReadWriteTx(["globalCurrencyExchanges"], async (tx) => {
-        const key = [req.currency, req.exchangeBaseUrl, req.exchangeMasterPub];
-        const existingRec =
-          await tx.globalCurrencyExchanges.indexes.byCurrencyAndUrlAndPub.get(
-            key,
-          );
-        if (!existingRec) {
-          return;
-        }
-        wex.ws.exchangeCache.clear();
-        checkDbInvariant(!!existingRec.id);
-        await tx.globalCurrencyExchanges.delete(existingRec.id);
-      });
+      await wex.db.runReadWriteTx(
+        { storeNames: ["globalCurrencyExchanges"] },
+        async (tx) => {
+          const key = [
+            req.currency,
+            req.exchangeBaseUrl,
+            req.exchangeMasterPub,
+          ];
+          const existingRec =
+            await tx.globalCurrencyExchanges.indexes.byCurrencyAndUrlAndPub.get(
+              key,
+            );
+          if (!existingRec) {
+            return;
+          }
+          wex.ws.exchangeCache.clear();
+          checkDbInvariant(!!existingRec.id, `no global exchange for ${j2s(key)}`);
+          await tx.globalCurrencyExchanges.delete(existingRec.id);
+        },
+      );
       return {};
     }
     case WalletApiOperation.AddGlobalCurrencyAuditor: {
       const req = codecForAddGlobalCurrencyAuditorRequest().decode(payload);
-      await wex.db.runReadWriteTx(["globalCurrencyAuditors"], async (tx) => {
-        const key = [req.currency, req.auditorBaseUrl, req.auditorPub];
-        const existingRec =
-          await tx.globalCurrencyAuditors.indexes.byCurrencyAndUrlAndPub.get(
-            key,
-          );
-        if (existingRec) {
-          return;
-        }
-        await tx.globalCurrencyAuditors.add({
-          currency: req.currency,
-          auditorBaseUrl: req.auditorBaseUrl,
-          auditorPub: req.auditorPub,
-        });
-        wex.ws.exchangeCache.clear();
-      });
+      await wex.db.runReadWriteTx(
+        { storeNames: ["globalCurrencyAuditors"] },
+        async (tx) => {
+          const key = [req.currency, req.auditorBaseUrl, req.auditorPub];
+          const existingRec =
+            await tx.globalCurrencyAuditors.indexes.byCurrencyAndUrlAndPub.get(
+              key,
+            );
+          if (existingRec) {
+            return;
+          }
+          await tx.globalCurrencyAuditors.add({
+            currency: req.currency,
+            auditorBaseUrl: req.auditorBaseUrl,
+            auditorPub: req.auditorPub,
+          });
+          wex.ws.exchangeCache.clear();
+        },
+      );
       return {};
     }
+    case WalletApiOperation.TestingWaitTasksDone: {
+      await waitTasksDone(wex);
+      return {};
+    }
+    case WalletApiOperation.TestingResetAllRetries:
+      await retryAll(wex);
+      return {};
     case WalletApiOperation.RemoveGlobalCurrencyAuditor: {
       const req = codecForRemoveGlobalCurrencyAuditorRequest().decode(payload);
-      await wex.db.runReadWriteTx(["globalCurrencyAuditors"], async (tx) => {
-        const key = [req.currency, req.auditorBaseUrl, req.auditorPub];
-        const existingRec =
-          await tx.globalCurrencyAuditors.indexes.byCurrencyAndUrlAndPub.get(
-            key,
-          );
-        if (!existingRec) {
-          return;
-        }
-        checkDbInvariant(!!existingRec.id);
-        await tx.globalCurrencyAuditors.delete(existingRec.id);
-        wex.ws.exchangeCache.clear();
-      });
+      await wex.db.runReadWriteTx(
+        { storeNames: ["globalCurrencyAuditors"] },
+        async (tx) => {
+          const key = [req.currency, req.auditorBaseUrl, req.auditorPub];
+          const existingRec =
+            await tx.globalCurrencyAuditors.indexes.byCurrencyAndUrlAndPub.get(
+              key,
+            );
+          if (!existingRec) {
+            return;
+          }
+          checkDbInvariant(!!existingRec.id, `no global currency for ${j2s(key)}`);
+          await tx.globalCurrencyAuditors.delete(existingRec.id);
+          wex.ws.exchangeCache.clear();
+        },
+      );
       return {};
     }
     case WalletApiOperation.ImportDb: {
@@ -1380,6 +1497,10 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       await applyDevExperiment(wex, req.devExperimentUri);
       return {};
     }
+    case WalletApiOperation.Shutdown: {
+      wex.ws.stop();
+      return {};
+    }
     case WalletApiOperation.GetVersion: {
       return getVersion(wex);
     }
@@ -1390,7 +1511,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
     case WalletApiOperation.TestingSetTimetravel: {
       const req = codecForTestingSetTimetravelRequest().decode(payload);
       setDangerousTimetravel(req.offsetMs);
-      wex.taskScheduler.reload();
+      await wex.taskScheduler.reload();
       return {};
     }
     case WalletApiOperation.DeleteExchange: {
@@ -1402,6 +1523,12 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       const req = codecForGetExchangeResourcesRequest().decode(payload);
       return await getExchangeResources(wex, req.exchangeBaseUrl);
     }
+    case WalletApiOperation.CanonicalizeBaseUrl: {
+      const req = codecForCanonicalizeBaseUrlRequest().decode(payload);
+      return {
+        url: canonicalizeBaseUrl(req.url),
+      };
+    }
     case WalletApiOperation.TestingInfiniteTransactionLoop: {
       const myDelayMs = (payload as any).delayMs ?? 5;
       const shouldFetch = !!(payload as any).shouldFetch;
@@ -1421,7 +1548,7 @@ async function dispatchRequestInternal<Op extends WalletApiOperation>(
       let loopCount = 0;
       while (true) {
         logger.info(`looping test write tx, iteration ${loopCount}`);
-        await wex.db.runReadWriteTx(["config"], async (tx) => {
+        await wex.db.runReadWriteTx({ storeNames: ["config"] }, async (tx) => {
           await tx.config.put({
             key: ConfigRecordKey.TestLoopTx,
             value: loopCount,
@@ -1468,7 +1595,7 @@ export function getObservedWalletExecutionContext(
   ws: InternalWalletState,
   cancellationToken: CancellationToken,
   oc: ObservabilityContext,
-) {
+): WalletExecutionContext {
   const wex: WalletExecutionContext = {
     ws,
     cancellationToken,
@@ -1485,7 +1612,7 @@ export function getNormalWalletExecutionContext(
   ws: InternalWalletState,
   cancellationToken: CancellationToken,
   oc: ObservabilityContext,
-) {
+): WalletExecutionContext {
   const wex: WalletExecutionContext = {
     ws,
     cancellationToken,
@@ -1512,6 +1639,14 @@ async function handleCoreApiRequest(
   id: string,
   payload: unknown,
 ): Promise<CoreApiResponse> {
+  if (operation !== WalletApiOperation.InitWallet) {
+    if (!ws.initCalled) {
+      throw Error("init must be called first");
+    }
+  }
+
+  await ws.ensureWalletDbOpen();
+
   let wex: WalletExecutionContext;
   let oc: ObservabilityContext;
 
@@ -1600,6 +1735,7 @@ export function applyRunConfigDefaults(
       skipDefaults: wcp?.testing?.skipDefaults ?? false,
       emitObservabilityEvents: wcp?.testing?.emitObservabilityEvents ?? false,
     },
+    lazyTaskLoop: wcp?.lazyTaskLoop ?? false,
   };
 }
 
@@ -1648,15 +1784,6 @@ export class Wallet {
     return this.ws.addNotificationListener(f);
   }
 
-  stop(): void {
-    this.ws.stop();
-  }
-
-  async runTaskLoop(opts?: RetryLoopOpts): Promise<void> {
-    await this.ws.ensureWalletDbOpen();
-    return this.ws.taskScheduler.run(opts);
-  }
-
   async handleCoreApiRequest(
     operation: string,
     id: string,
@@ -1709,6 +1836,34 @@ export class Cache<T> {
   }
 }
 
+/**
+ * Implementation of triggers for the wallet DB.
+ */
+class WalletDbTriggerSpec implements TriggerSpec {
+  constructor(public ws: InternalWalletState) {}
+
+  afterCommit(info: AfterCommitInfo): void {
+    if (info.mode !== "readwrite") {
+      return;
+    }
+    logger.trace(
+      `in after commit callback for readwrite, modified ${j2s([
+        ...info.modifiedStores,
+      ])}`,
+    );
+    const modified = info.accessedStores;
+    if (
+      modified.has(WalletStoresV1.exchanges.storeName) ||
+      modified.has(WalletStoresV1.exchangeDetails.storeName) ||
+      modified.has(WalletStoresV1.denominations.storeName) ||
+      modified.has(WalletStoresV1.globalCurrencyAuditors.storeName) ||
+      modified.has(WalletStoresV1.globalCurrencyExchanges.storeName)
+    ) {
+      this.ws.clearAllCaches();
+    }
+  }
+}
+
 /**
  * Internal state of the wallet.
  *
@@ -1755,15 +1910,19 @@ export class InternalWalletState {
 
   private _config: Readonly<WalletRunConfig> | undefined;
 
-  private _db: DbAccess<typeof WalletStoresV1> | undefined = undefined;
+  private _indexedDbHandle: IDBDatabase | undefined = undefined;
+
+  private _dbAccessHandle: DbAccess<typeof WalletStoresV1> | undefined;
 
   private _http: HttpRequestLibrary | undefined = undefined;
 
   get db(): DbAccess<typeof WalletStoresV1> {
-    if (!this._db) {
-      throw Error("db not initialized");
+    if (!this._dbAccessHandle) {
+      this._dbAccessHandle = this.createDbAccessHandle(
+        CancellationToken.CONTINUE,
+      );
     }
-    return this._db;
+    return this._dbAccessHandle;
   }
 
   devExperimentState: DevExperimentState = {};
@@ -1779,8 +1938,6 @@ export class InternalWalletState {
   initWithConfig(newConfig: WalletRunConfig): void {
     this._config = newConfig;
 
-    logger.info(`setting new config to ${j2s(newConfig)}`);
-
     this._http = this.httpFactory(newConfig);
 
     if (this.config.testing.devModeActive) {
@@ -1788,6 +1945,20 @@ export class InternalWalletState {
     }
   }
 
+  createDbAccessHandle(
+    cancellationToken: CancellationToken,
+  ): DbAccess<typeof WalletStoresV1> {
+    if (!this._indexedDbHandle) {
+      throw Error("db not initialized");
+    }
+    return new DbAccessImpl(
+      this._indexedDbHandle,
+      WalletStoresV1,
+      new WalletDbTriggerSpec(this),
+      cancellationToken,
+    );
+  }
+
   get config(): WalletRunConfig {
     if (!this._config) {
       throw Error("config not initialized");
@@ -1814,7 +1985,7 @@ export class InternalWalletState {
   }
 
   async ensureWalletDbOpen(): Promise<void> {
-    if (this._db) {
+    if (this._indexedDbHandle) {
       return;
     }
     const myVersionChange = async (): Promise<void> => {
@@ -1822,7 +1993,7 @@ export class InternalWalletState {
     };
     try {
       const myDb = await openTalerDatabase(this.idb, myVersionChange);
-      this._db = myDb;
+      this._indexedDbHandle = myDb;
     } catch (e) {
       logger.error("error writing to database during initialization");
       throw TalerError.fromDetail(TalerErrorCode.WALLET_DB_UNAVAILABLE, {
@@ -1859,6 +2030,9 @@ export class InternalWalletState {
     this.stopped = true;
     this.timerGroup.stopCurrentAndFutureTimers();
     this.cryptoDispatcher.stop();
+    this.taskScheduler.shutdown().catch((e) => {
+      logger.warn(`shutdown failed: ${safeStringifyException(e)}`);
+    });
   }
 
   /**
@@ -1893,7 +2067,7 @@ export class InternalWalletState {
     } finally {
       for (const token of tokens) {
         this.resourceLocks.delete(token);
-        let waiter = (this.resourceWaiters[token] ?? []).shift();
+        const waiter = (this.resourceWaiters[token] ?? []).shift();
         if (waiter) {
           waiter.resolve();
         }
