@@ -52,7 +52,7 @@ const Body = styled.div`
 
 export function Modal({ title, children, onClose }: Props): VNode {
   return (
-    <div style={{ position: "fixed", top: 0, width: "100%", height: "100%" }}>
+    <div style={{ top: 0, width: "100%", height: "100%" }}>
 
       <FullSize onClick={onClose?.onClick}>
         <div
@@ -64,6 +64,7 @@ export function Modal({ title, children, onClose }: Props): VNode {
             margin: "auto",
             borderRadius: 8,
             padding: 8,
+            zIndex: 100,
             // overflow: "scroll",
           }}
         >
