@@ -23,13 +23,24 @@ import { useAsyncAsHook } from "../../hooks/useAsyncAsHook.js";
 import { Props, State } from "./index.js";
 import { buildTermsOfServiceState } from "./utils.js";
 
-export function useComponentState({ exchangeUrl, onChange }: Props): State {
+const supportedFormats = {
+  "text/html": "HTML",
+  "text/xml" : "XML",
+  "text/markdown" : "Markdown",
+  "text/plain" : "Plain text",
+  "text/pdf" : "PDF",
+}
+
+export function useComponentState({ showEvenIfaccepted, exchangeUrl, readOnly, children }: Props): State {
   const api = useBackendContext();
-  const readOnly = !onChange;
-  const [showContent, setShowContent] = useState<boolean>(readOnly);
-  const { i18n } = useTranslationContext();
+  const [showContent, setShowContent] = useState<boolean>(!!readOnly);
+  const { i18n, lang } = useTranslationContext();
+  const [tosLang, setTosLang] = useState<string>()
   const { pushAlertOnError } = useAlertContext();
 
+  const [format, setFormat] = useState("text/html")
+
+  const acceptedLang = tosLang ?? lang
   /**
    * For the exchange selected, bring the status of the terms of service
    */
@@ -38,14 +49,20 @@ export function useComponentState({ exchangeUrl, onChange }: Props): State {
       WalletApiOperation.GetExchangeTos,
       {
         exchangeBaseUrl: exchangeUrl,
-        acceptedFormat: ["text/xml"],
+        acceptedFormat: [format],
+        acceptLanguage: acceptedLang,
       },
     );
 
+    const supportedLangs = exchangeTos.tosAvailableLanguages.reduce((prev, cur) => {
+      prev[cur] = cur
+      return prev;
+    }, {} as Record<string, string>)
+
     const state = buildTermsOfServiceState(exchangeTos);
 
-    return { state };
-  }, []);
+    return { state, supportedLangs };
+  }, [acceptedLang, format]);
 
   if (!terms) {
     return {
@@ -57,12 +74,13 @@ export function useComponentState({ exchangeUrl, onChange }: Props): State {
     return {
       status: "error",
       error: alertFromError(
+        i18n,
         i18n.str`Could not load the status of the term of service`,
         terms,
       ),
     };
   }
-  const { state } = terms.response;
+  const { state, supportedLangs } = terms.response;
 
   async function onUpdate(accepted: boolean): Promise<void> {
     if (!state) return;
@@ -70,17 +88,11 @@ export function useComponentState({ exchangeUrl, onChange }: Props): State {
     if (accepted) {
       await api.wallet.call(WalletApiOperation.SetExchangeTosAccepted, {
         exchangeBaseUrl: exchangeUrl,
-        etag: state.version,
       });
     } else {
       // mark as not accepted
-      await api.wallet.call(WalletApiOperation.SetExchangeTosAccepted, {
-        exchangeBaseUrl: exchangeUrl,
-        etag: undefined,
-      });
     }
-    // setAccepted(accepted);
-    if (!readOnly) onChange(accepted); //external update
+    terms?.retry()
   }
 
   const accepted = state.status === "accepted";
@@ -88,9 +100,9 @@ export function useComponentState({ exchangeUrl, onChange }: Props): State {
   const base = {
     error: undefined,
     showingTermsOfService: {
-      value: showContent,
+      value: showContent && (!accepted || showEvenIfaccepted),
       button: {
-        onClick: pushAlertOnError(async () => {
+        onClick: accepted && !showEvenIfaccepted ? undefined : pushAlertOnError(async () => {
           setShowContent(!showContent);
         }),
       },
@@ -99,7 +111,7 @@ export function useComponentState({ exchangeUrl, onChange }: Props): State {
     termsAccepted: {
       value: accepted,
       button: {
-        onClick: pushAlertOnError(async () => {
+        onClick: readOnly ? undefined : pushAlertOnError(async () => {
           const newValue = !accepted; //toggle
           await onUpdate(newValue);
           setShowContent(false);
@@ -108,25 +120,41 @@ export function useComponentState({ exchangeUrl, onChange }: Props): State {
     },
   };
 
-  if (showContent) {
+  if (accepted) {
+    return {
+      status: "show-buttons-accepted",
+      ...base,
+      children,
+    };
+  }
+
+  if ((accepted && showEvenIfaccepted) || showContent) {
     return {
       status: "show-content",
       error: undefined,
       terms: state,
       showingTermsOfService: readOnly ? undefined : base.showingTermsOfService,
-      termsAccepted: readOnly ? undefined : base.termsAccepted,
+      termsAccepted: base.termsAccepted,
+      tosFormat: {
+        onChange: pushAlertOnError(async (s) => {
+          setFormat(s)
+        }),
+        list: supportedFormats,
+        value: format ?? ""
+      },
+      tosLang: {
+        onChange: pushAlertOnError(async (s) => {
+          setTosLang(s)
+        }),
+        list: supportedLangs,
+        value: tosLang ?? lang
+      }
     };
   }
   //showing buttons
-  if (accepted) {
-    return {
-      status: "show-buttons-accepted",
-      ...base,
-    };
-  } else {
-    return {
-      status: "show-buttons-not-accepted",
-      ...base,
-    };
-  }
+  return {
+    status: "show-buttons-not-accepted",
+    ...base,
+  };
+
 }
