@@ -50,7 +50,8 @@ export function useComponentState({
     return {
       status: "error",
       error: alertFromError(
-        i18n.str`Could not load the status of the term of service`,
+        i18n,
+        i18n.str`Could not load the list of exchanges`,
         hook,
       ),
     };
@@ -103,7 +104,8 @@ export function useComponentState({
       return {
         status: "error",
         error: alertFromError(
-          i18n.str`Could not load the status of the term of service`,
+          i18n,
+          i18n.str`Could not load the invoice status`,
           hook,
         ),
       };
@@ -166,8 +168,8 @@ export function useComponentState({
           subject === undefined
             ? undefined
             : !subject
-            ? "Can't be empty"
-            : undefined,
+              ? "Can't be empty"
+              : undefined,
         value: subject ?? "",
         onInput: pushAlertOnError(async (e) => setSubject(e)),
       },
