@@ -197,7 +197,11 @@ async function sendMessageToBackground<Op extends keyof ExtensionOperations>(
     let timedout = false;
     const timerId = setTimeout(() => {
       timedout = true;
-      reject(TalerError.fromDetail(TalerErrorCode.GENERIC_TIMEOUT, {}))
+      reject(TalerError.fromDetail(TalerErrorCode.GENERIC_TIMEOUT, {
+        requestMethod: "wallet",
+        requestUrl: message.operation,
+        timeoutMs: 20 * 1000,
+      }))
     }, 20 * 1000); //five seconds
     try {
       chrome.runtime.sendMessage(messageWithId, (backgroundResponse) => {
