@@ -17,25 +17,31 @@
 import {
   AbsoluteTime,
   Amounts,
-  CoinDumpJson,
   CoinStatus,
-  ExchangeListItem,
   ExchangeTosStatus,
   LogLevel,
   NotificationType,
   ScopeType,
-  parseWithdrawUri,
-  stringifyWithdrawExchange,
+  stringifyWithdrawExchange
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useRef, useState } from "preact/hooks";
+import { Pages } from "../NavigationBar.js";
 import { Checkbox } from "../components/Checkbox.js";
 import { SelectList } from "../components/SelectList.js";
 import { Time } from "../components/Time.js";
-import { DestructiveText, LinkPrimary, NotifyUpdateFadeOut, SubTitle, SuccessText, WarningText } from "../components/styled/index.js";
+import { ActiveTasksTable } from "../components/WalletActivity.js";
+import {
+  DestructiveText,
+  LinkPrimary,
+  NotifyUpdateFadeOut,
+  SubTitle,
+  SuccessText,
+  WarningText,
+} from "../components/styled/index.js";
 import { useAlertContext } from "../context/alert.js";
 import { useBackendContext } from "../context/backend.js";
 import { useAsyncAsHook } from "../hooks/useAsyncAsHook.js";
@@ -44,11 +50,7 @@ import { Button } from "../mui/Button.js";
 import { Grid } from "../mui/Grid.js";
 import { Paper } from "../mui/Paper.js";
 import { TextField } from "../mui/TextField.js";
-import { Pages } from "../NavigationBar.js";
-import { CoinInfo } from "@gnu-taler/taler-wallet-core/dbless";
-import { ActiveTasksTable } from "../components/WalletActivity.js";
 
-type CoinsInfo = CoinDumpJson["coins"];
 type CalculatedCoinfInfo = {
   // ageKeysCount: number | undefined;
   denom_value: number;
@@ -64,15 +66,7 @@ type SplitedCoinInfo = {
   usable: CalculatedCoinfInfo[];
 };
 
-export interface Props {
-  // FIXME: Pending operations don't exist anymore.
-}
-
-function hashObjectId(o: any): string {
-  return JSON.stringify(o);
-}
-
-export function DeveloperPage({ }: Props): VNode {
+export function DeveloperPage(): VNode {
   const { i18n } = useTranslationContext();
   const [downloadedDatabase, setDownloadedDatabase] = useState<
     { time: Date; content: string } | undefined
@@ -110,8 +104,8 @@ export function DeveloperPage({ }: Props): VNode {
 
   useEffect(() => {
     return api.listener.onUpdateNotification(listenAllEvents, (ev) => {
-      console.log("event", ev)
-      return hook?.retry()
+      console.log("event", ev);
+      return hook?.retry();
     });
   });
 
@@ -275,7 +269,6 @@ export function DeveloperPage({ }: Props): VNode {
         })}
       />
 
-
       <SubTitle>
         <i18n.Translate>Exchange Entries</i18n.Translate>
       </SubTitle>
@@ -336,19 +329,32 @@ export function DeveloperPage({ }: Props): VNode {
                       );
                   }
                 }
-                const uri = !e.masterPub ? undefined : stringifyWithdrawExchange({
-                  exchangeBaseUrl: e.exchangeBaseUrl,
-                  exchangePub: e.masterPub,
-                });
+                const uri = !e.masterPub
+                  ? undefined
+                  : stringifyWithdrawExchange({
+                      exchangeBaseUrl: e.exchangeBaseUrl,
+                    });
                 return (
                   <tr key={idx}>
                     <td>
                       <a href={!uri ? undefined : Pages.defaultCta({ uri })}>
-                        {e.scopeInfo ? `${e.scopeInfo.currency} (${e.scopeInfo.type === ScopeType.Global ? "global" : "regional"})` : e.currency}
+                        {e.scopeInfo
+                          ? `${e.scopeInfo.currency} (${
+                              e.scopeInfo.type === ScopeType.Global
+                                ? "global"
+                                : "regional"
+                            })`
+                          : e.currency}
                       </a>
                     </td>
                     <td>
-                      <a href={new URL(`/keys`, e.exchangeBaseUrl).href} target="_blank">{e.exchangeBaseUrl}</a>
+                      <a
+                        href={new URL(`/keys`, e.exchangeBaseUrl).href}
+                        target="_blank"
+                        rel="noreferrer" 
+                      >
+                        {e.exchangeBaseUrl}
+                      </a>
                     </td>
                     <td>
                       {e.exchangeEntryStatus} / {e.exchangeUpdateStatus}
@@ -359,10 +365,10 @@ export function DeveloperPage({ }: Props): VNode {
                     <td>
                       {e.lastUpdateTimestamp
                         ? AbsoluteTime.toIsoString(
-                          AbsoluteTime.fromPreciseTimestamp(
-                            e.lastUpdateTimestamp,
-                          ),
-                        )
+                            AbsoluteTime.fromPreciseTimestamp(
+                              e.lastUpdateTimestamp,
+                            ),
+                          )
                         : "never"}
                     </td>
                     <td>
@@ -381,31 +387,25 @@ export function DeveloperPage({ }: Props): VNode {
                       </button>
                       <button
                         onClick={() => {
-                          api.wallet.call(
-                            WalletApiOperation.DeleteExchange,
-                            {
-                              exchangeBaseUrl: e.exchangeBaseUrl,
-                            },
-                          );
+                          api.wallet.call(WalletApiOperation.DeleteExchange, {
+                            exchangeBaseUrl: e.exchangeBaseUrl,
+                          });
                         }}
                       >
                         Delete
                       </button>
                       <button
                         onClick={() => {
-                          api.wallet.call(
-                            WalletApiOperation.DeleteExchange,
-                            {
-                              exchangeBaseUrl: e.exchangeBaseUrl,
-                              purge: true,
-                            },
-                          );
+                          api.wallet.call(WalletApiOperation.DeleteExchange, {
+                            exchangeBaseUrl: e.exchangeBaseUrl,
+                            purge: true,
+                          });
                         }}
                       >
                         Purge
                       </button>
-                      {e.scopeInfo && e.masterPub && e.currency ?
-                        (e.scopeInfo.type === ScopeType.Global ?
+                      {e.scopeInfo && e.masterPub && e.currency ? (
+                        e.scopeInfo.type === ScopeType.Global ? (
                           <button
                             onClick={() => {
                               api.wallet.call(
@@ -418,30 +418,27 @@ export function DeveloperPage({ }: Props): VNode {
                               );
                             }}
                           >
-
                             Make regional
                           </button>
-                          : e.scopeInfo.type === ScopeType.Auditor ?
-                            undefined
-
-                            : e.scopeInfo.type === ScopeType.Exchange ?
-                              <button
-                                onClick={() => {
-                                  api.wallet.call(
-                                    WalletApiOperation.AddGlobalCurrencyExchange,
-                                    {
-                                      exchangeBaseUrl: e.exchangeBaseUrl,
-                                      currency: e.currency!,
-                                      exchangeMasterPub: e.masterPub!,
-                                    },
-                                  );
-                                }}
-                              >
-
-                                Make global
-                              </button>
-                              : undefined) : undefined
-                      }
+                        ) : e.scopeInfo.type ===
+                          ScopeType.Auditor ? undefined : e.scopeInfo.type ===
+                          ScopeType.Exchange ? (
+                          <button
+                            onClick={() => {
+                              api.wallet.call(
+                                WalletApiOperation.AddGlobalCurrencyExchange,
+                                {
+                                  exchangeBaseUrl: e.exchangeBaseUrl,
+                                  currency: e.currency!,
+                                  exchangeMasterPub: e.masterPub!,
+                                },
+                              );
+                            }}
+                          >
+                            Make global
+                          </button>
+                        ) : undefined
+                      ) : undefined}
                       <button
                         onClick={() => {
                           api.wallet.call(
@@ -469,7 +466,6 @@ export function DeveloperPage({ }: Props): VNode {
         </LinkPrimary>
       </div>
 
-
       <Paper style={{ padding: 10, margin: 10 }}>
         <h3>Logging</h3>
         <div>
