@@ -242,53 +242,56 @@ function TransactionTemplate({
             )}
           />
         ) : undefined}
-        {transaction.txState.minor === TransactionMinorState.KycRequired && (
-          <AlertView
-            alert={{
-              type: "warning",
-              message: i18n.str`KYC check required for the transaction to complete.`,
-              description:
-                transaction.kycUrl && typeof transaction.kycUrl === "string" ? (
-                  <div>
-                    <i18n.Translate>
-                      Follow this link to the{` `}
-                      <a
-                        rel="noreferrer"
-                        target="_bank"
-                        href={transaction.kycUrl}
-                      >
-                        KYC verifier.
-                      </a>
-                    </i18n.Translate>
-                  </div>
-                ) : (
-                  i18n.str`No additional information has been provided.`
-                ),
-            }}
-          />
-        )}
-        {transaction.txState.minor === TransactionMinorState.AmlRequired && (
-          <WarningBox>
-            <i18n.Translate>
-              The transaction has been blocked since the account required an AML
-              check.
-            </i18n.Translate>
-          </WarningBox>
-        )}
-        {transaction.txState.major === TransactionMajorState.Pending && (
-          <WarningBox>
-            <div style={{ justifyContent: "center", lineHeight: "25px" }}>
-              <i18n.Translate>This transaction is not completed</i18n.Translate>
-              <Link onClick={onRetry} style={{ padding: 0 }}>
-                <SvgIcon
-                  title={i18n.str`Retry`}
-                  dangerouslySetInnerHTML={{ __html: refreshIcon }}
-                  color="black"
-                />
-              </Link>
-            </div>
-          </WarningBox>
-        )}
+        {transaction.txState.major === TransactionMajorState.Pending &&
+          (transaction.txState.minor === TransactionMinorState.KycRequired ? (
+            <AlertView
+              alert={{
+                type: "warning",
+                message: i18n.str`KYC check required for the transaction to complete.`,
+                description:
+                  transaction.kycUrl &&
+                  typeof transaction.kycUrl === "string" ? (
+                    <div>
+                      <i18n.Translate>
+                        Follow this link to the{` `}
+                        <a
+                          rel="noreferrer"
+                          target="_bank"
+                          href={transaction.kycUrl}
+                        >
+                          KYC verifier.
+                        </a>
+                      </i18n.Translate>
+                    </div>
+                  ) : (
+                    i18n.str`No additional information has been provided.`
+                  ),
+              }}
+            />
+          ) : transaction.txState.minor ===
+            TransactionMinorState.AmlRequired ? (
+            <WarningBox>
+              <i18n.Translate>
+                The transaction has been blocked since the account required an
+                AML check.
+              </i18n.Translate>
+            </WarningBox>
+          ) : (
+            <WarningBox>
+              <div style={{ justifyContent: "center", lineHeight: "25px" }}>
+                <i18n.Translate>
+                  This transaction is not completed
+                </i18n.Translate>
+                <Link onClick={onRetry} style={{ padding: 0 }}>
+                  <SvgIcon
+                    title={i18n.str`Retry`}
+                    dangerouslySetInnerHTML={{ __html: refreshIcon }}
+                    color="black"
+                  />
+                </Link>
+              </div>
+            </WarningBox>
+          ))}
         {transaction.txState.major === TransactionMajorState.Aborted && (
           <InfoBox>
             <i18n.Translate>This transaction was aborted.</i18n.Translate>
@@ -1139,9 +1142,6 @@ export function TransactionView({
   if (transaction.type === TransactionType.Recoup) {
     throw Error("recoup transaction not implemented");
   }
-  if (transaction.type === TransactionType.Reward) {
-    throw Error("recoup transaction not implemented");
-  }
   assertUnreachable(transaction);
 }
 
